      subroutine isexdic2 ( ni, nj, nvirtual, rhside, vsum,
     +                       alphad, iarea, iindsc, nintpol,
     +                       dij, mct, intpnr, ndegfd, solut,
     +                       invjac )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.1    date 27-08-1999 Duncan: move invjac to distribution
c                                       routines
c        version 1.0    date 29-10-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribution of interpolation or finite difference formulas
c    over integration point with respect to the convection term
c    Non-conservative scheme for explicit convection
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iindsc(*), alphad, intpnr,
     +        iarea(2,2), nintpol(2,5), dij(2,4,2,5), ndegfd,
     +        mct(2,2,-2:2,-2:2,-2:2)
      double precision vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 solut(*), rhside(ni+1,nj+1,2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual))

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dij       i    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates
c                    dij(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     iarea     i    Area of arrays
c     iindsc    i    reserved
c     intpnr    i    Integration point under consideration
c     invjac    i    1/sqrt(g) in pnttyp
c     mct       i    Integer matrix information
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nintpol   i    Number of interpolation points in the distribution
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    i    reserved
c     solut     i    reserved
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, kdir, iconct, dcell(2)
      double precision fact

c     dcell     reserved
c     fact      Multiplication factor
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     kdir      General loop variable for 1...ndim loop
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISEXUP2   Perform a vector update for ISNaS over a 2-dimensional region
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    This is the main distribution subroutine for the convection
c    This subroutine is called in a loop over the integration points and
c    in the case of a momentum equation also in a loop over the unknowns
c    The contribution of the integration point to the molecule has been stored
c    in the array vsum which has the length of all points in the mesh
c    including the virtual points
c    This array is built as if the integration point has a direct contribution
c    to the right hand side.
c    However, in general the integration point does not contain the
c    unknown to which the solution vector corresponds.
c
c    For example consider the V1 component in the V1 row of the convection
c    stencil in R2
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c    One of the contributions to the molecule is the term
c
c    rho/sqrt(g) V2 in the integration point i,j+1/2
c
c    This contribution has been computed before and stored in array vsum
c
c    However in point i,j+1/2 there is no V1 unknown and this contribution
c    must be distributed over the two surrounding points i,j and i,j+1
c
c    In this subroutine the distribution takes place and the corresponding
c    contributions (rho/sqrt(g) V2 )/2 are multiplied by the velocities in the
c    correct positions
c
c    The following tables and parameters define how the distribution must be
c    carried out:
c
c    dcell contains the global coordinate distance from integration point
c          to the centre of the integration cell (in the example (0,1))
c    dij   contains the global coordinate distance from the distribution points
c          to the centre of the integration cell (in the example (0,1))
c    nintpol contains the number of distribution points
c    mct   contains the column number in the matrix where the value
c          must be stored
c
c                    PSEUDO CODE
c
c
c    For each velocity component in the integration point do
c       fact := 1/(number of distribution points)
c       for i := 1 (1) number of distribution points do
c          Compute position in stencil with which velocity
c          the coefficient must be multiplied
c          Add vsum * solution to the right hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Initialize shifts

      dcell(1) = 0
      dcell(2) = 0

      if ( intpnr.eq.0 ) then

c     --- Cell centre (0,0), no shift required

         do kdir = 1, 2

c        --- Loop over points in interpolation formula

            fact = 1d0/nintpol(kdir,1)
            do i = 1, nintpol(kdir,1)

c           --- determine corresponding location in the stencil

               iconct = mct(alphad,kdir,dij(kdir,i,1,1),
     +                      dij(kdir,i,2,1),0)

c              --- distribute vsum

               call isexup2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                          rhside(1,1,alphad), i,
     +                          fact, vsum(1,kdir), iarea, alphad,
     +                          iconct, solut, ndegfd, iindsc(5), 1,
     +                          invjac)
            end do

         end do

      else if ( intpnr.eq.1 ) then

c     --- If V^1 cell then (-1,0) and (1,0)
c         If V^2 cell then (0,-1) and (0,1)

         kdir = alphad

c        --- Loop over points in interpolation formula
c            First left or under point

         dcell(alphad) = -1
         fact = -1d0/nintpol(kdir,2)
         do i = 1, nintpol(kdir,2)

c        --- determine corresponding location in the stencil

            iconct = mct(alphad,kdir,dij(kdir,i,1,2),
     +                   dij(kdir,i,2,2),0)

c           --- distribute vsum

            call isexup2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                       rhside(1,1,alphad), i,
     +                       fact, vsum(1,kdir), iarea, alphad,
     +                       iconct, solut, ndegfd, iindsc(5), 2,
     +                       invjac)
         end do

c        --- Next right or upper point

         dcell(alphad) = 0
         fact = -fact
         do i = 1, nintpol(kdir,3)

c        --- determine corresponding location in the stencil

            iconct = mct(alphad,kdir,dij(kdir,i,1,3),
     +                   dij(kdir,i,2,3),0)

c           --- distribute vsum

            call isexup2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                       rhside(1,1,alphad), i,
     +                       fact, vsum(1,kdir), iarea, alphad,
     +                       iconct, solut, ndegfd, iindsc(5), 3,
     +                       invjac)
         end do

      else

c     --- If V^1 cell then (0,-1) and (0,1)
c         If V^2 cell then (-1,0) and (1,0)

         do kdir = 1, 2

c        --- Loop over points in interpolation formula
c            First left or under point

            dcell(3-alphad) = 0
            fact = -1d0/nintpol(kdir,4)
            do i = 1, nintpol(kdir,4)

c           --- determine corresponding location in the stencil

               iconct = mct(alphad,kdir,dij(kdir,i,1,4),
     +                      dij(kdir,i,2,4),0)

c              --- distribute vsum

               call isexup2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                          rhside(1,1,alphad), i,
     +                          fact, vsum(1,kdir), iarea, alphad,
     +                          iconct, solut, ndegfd, iindsc(5), 4,
     +                          invjac)
            end do

c           --- Next right or upper point

            dcell(3-alphad) = 1
            fact = -fact
            do i = 1, nintpol(kdir,5)

c           --- determine corresponding location in the stencil

               iconct = mct(alphad,kdir,dij(kdir,i,1,5),
     +                      dij(kdir,i,2,5),0)

c              --- distribute vsum

               call isexup2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                          rhside(1,1,alphad), i,
     +                          fact, vsum(1,kdir), iarea, alphad,
     +                          iconct, solut, ndegfd, iindsc(5), 5,
     +                          invjac)
            end do

         end do

      end if

      end
