      subroutine isomegb3( nside   , i     , j     , bndcon, coefs  ,
     +                     coor    , solut , rintur, ndir  ,
     +                     t1dir   , t2dir , ni    , nj    , nk     ,
     +                     nvirtual, ncoefs, maxb  , eddy  , viscorr,
     +                     nsign   , it1lo , it2lo , it1hi , it2hi   )
c ======================================================================
c
c        programmer     Marcel Zijlema
c
c        version 1.0    date   21-02-1996 (extracted from isomegbc)
c
c
c   copyright (c) 1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fill BNDCON for the specific dissipation rate (omega) equation
c     with or without viscous corrections in R3
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nside, i, j, nvirtual, ni, nj, nk,
     +        ncoefs, maxb, ndir, t1dir,
     +        t2dir, nsign, it1lo, it1hi, it2lo, it2hi,
     +        viscorr
      double precision coor(*), rintur(*), solut(*),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ncoefs),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 bndcon(1:maxb,it1lo:it1hi,it2lo:it2hi)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     i         i    General loop variable
c     it1hi     i    Upper bound on first tangential index for destination
c     it1lo     i    Lower bound on first tangential index for destination
c     it2hi     i    Upper bound on second tangential index for destination
c     it2lo     i    Lower bound on second tangential index for destination
c     j         i    General loop variable
c     maxb      i    Maximum number of real values for one boundary condition
c     ncoefs    i    Number of coefficients
c     ndir      i    Direction of normal vector on this side
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nside     i    Number of boundary for which we need the
c                    information
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     nvirtual  i    Number of rows of virtual cells
c     rintur    i    reserved
c     solut     i    reserved
c     t1dir     i    Direction of first tangential vector on this side
c     t2dir     i    Direction of second tangential vector on this side
c     viscorr   i    Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp
      double precision ndist, physut, rhow, mu, nomeg, facs(1:2), yplus

c     facs      Multiplication factor
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     mu        The laminar viscosity
c     ndist     Distance normal to a solid wall
c     nomeg     A constant to determine the analytical solution of
c               omega near a solid wall (in viscous sublayer)
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     yplus     The dimensionless normal distance to the wall
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  auxiliary subroutine to iserrsub
c     ISERREAL  auxiliary subroutine to iserrsub
c     ISERRSUB  gives error messages
c     ISWLLQN3  Determine essential wall quantities
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c   1201  Y+ > 5, which means that the grid near the wall is not fine
c         enough. You must either increase a number of grid points normal
c         to the wall or the cell aspect ratio
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isomegb3')

      if ( nside.eq.1 ) then
        ip = 1
        jp = int((i+1)/2)
        kp = int((j+1)/2)
      else if ( nside.eq.2 ) then
        ip = ni
        jp = int((i+1)/2)
        kp = int((j+1)/2)
      else if ( nside.eq.3 ) then
        ip = int((j+1)/2)
        jp = 1
        kp = int((i+1)/2)
      else if ( nside.eq.4 ) then
        ip = int((j+1)/2)
        jp = nj
        kp = int((i+1)/2)
      else if ( nside.eq.5 ) then
        ip = int((i+1)/2)
        jp = int((j+1)/2)
        kp = 1
      else if ( nside.eq.6 ) then
        ip = int((i+1)/2)
        jp = int((j+1)/2)
        kp = nk
      end if
      if ( ip.gt.ni ) ip = ni
      if ( jp.gt.nj ) jp = nj
      if ( kp.gt.nk ) kp = nk

c     --- First, compute two essential wall quantities: distance
c         perpendicular to the wall and physical tangential velocity

      call iswllqn3( physut, ndist, facs, coor, solut, ni, nj, nk,
     +               nvirtual, ndir, ip, jp, kp, nside, nsign,
     +               t1dir, t2dir )

      rhow = coefs(ip,jp,kp,1)
      mu = coefs(ip,jp,kp,4)-rhow*eddy(ip,jp,kp)*rintur(19)
      yplus = sqrt(rhow*ndist*abs(physut)/mu)

      if ( yplus.gt.5d0 ) then
        call iserrchr( 'Y+', 1 )
        call iserreal( yplus, 1 )
        call iserrchr( 'Y+', 2 )
        call iserrsub ( 1201, 0, 1, 2 )
      end if

c     --- specific dissipation rate equation; fill BNDCON with or
c         without viscous corrections

      if ( viscorr.eq.0 ) then
        nomeg = 6d0/rintur(17)
      else if ( viscorr.eq.1 ) then
        nomeg = 2d0/rintur(16)
      end if

      bndcon(1,i,j) = nomeg*mu/(rhow*ndist*ndist)

      call iserclos('isomegb3')

      end
