      subroutine isbcs3n ( iface, alphad, ni, nj, nk,
     +                     ibinfor, ilow1, ihigh1, ilow2,
     +                     ihigh2, bndcon, prems, rhside )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 25-05-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Fill prescribed velocities in the term precm, which is used to build the
c   stress matrix. 3D WesBeek version
c   This subroutine is restricted to a normal cell with cell center
c   at one cell distance of the boundary
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    discretization
c    momentum_equation
c    stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, alphad, iface,
     +        ilow1, ihigh1, ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2)
      double precision rhside(ni+1,nj+1,nk+1,3),
     +                 prems(ni+1,nj+1,nk+1,3,6),
     +                 bndcon(3,ilow1:ihigh1,ilow2:ihigh2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     prems    i/o   Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy
      parameter( unut = 4, uxuy = 10 )
      integer i, j, k, l, tdir(2), iarea(3,2), mxp(3), gammad, btype
      double precision sum

c     btype     Type of boundary condition
c     gammad    Direction index
c     i         General loop variable
c     iarea     Area of arrays
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     mxp       Auxiliary array to calculate boundary point number
c     sum       Help variable to compute a sum
c     tdir      Direction of tangent vector on this side
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Array prems(i,j,k,beta,gamma) contains the contribution of the velocity
c    to the stress tensor for a specific integration point with respect to the
c    V^alphad cell with center (i,j,k) (global coordinates)
c    beta (1,2,3) refers to the components of the Cartesian vector
c    Each integration point is surrounded by 6 points that are used to
c    express the gradient in terms of the velocity vector
c    Gamma refers to these six points
c    With respect to alphad = 1, the local shift of a gamma point compared
c    to an integration point is (-1,0,0), (1,0,0), (0,-1,0), ... , (0,0,1)
c    For alphad = 2 and 3 a cyclic permutation is used hence, for alphad
c    the sequence is (0,-1,0), (0,1,0), (0,0,-1), ...
c
c    If the cell center is on the boundary and the velocity is prescribed,
c    then the unknown itself is eliminated
c    hence in this subroutine we only consider points on the boundary that
c    correspond to cells with their center not on the boundary
c    There are two such cells:
c    Cells corresponding to a normal component at one cell distance of the
c    boundary and cells corresponding to a tangential component at half a
c    cell distance of the boundary
c
c    For a normal cell at one cell distance of the boundary, there is only
c    one point that is on the boundary
c    For an alphad cell this point corresponds to the integration point
c    in the alphad direction and in the direction of the boundary
c    Of this integration point only the surrounding point
c    in the alphad direction and in the direction of the boundary is on the
c    boundary
c    Hence the integration point must satisfy locdx(alphad) = 1 for an even
c    face and locdx(alphad) = -1 for an odd face.
c    The corresponding point at the boundary refers to gammad = 1 for an even
c    face and gammad = -1 for an odd face.
c
c    In this subroutine only the normal cell is considered
c
c    For face 1 we have the following situation:
c
c       i=2, gamma=1, locdx(1)=-1, alphad=1
c       j=1(1)nj,  k=1(1)nk
c       point on boundary is (2j,2k) (in arrays bndcon and ibinfor)
c       rhside(i,j,k,alphad) = rhside(i,j,k,alphad) +
c                              prems(i,j,k,beta,gamma) * u_beta(boundary_point)
c
c    Pseudo code
c
c    Set under and upper indices for loop variables
c    Define tangential directions along the boundary (necessary for arrays
c    bndcon and ibinfor)
c    For all points on the boundary do
c       Find the type of boundary condition in the point
c       If ( velocity is prescribed in point ) then
c          update right-hand side
c          set corresponding value in prems equal to zero
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Set under and upper indices for loop variables

      iarea(1,1) = 1
      iarea(1,2) = ni
      iarea(2,1) = 1
      iarea(2,2) = nj
      iarea(3,1) = 1
      iarea(3,2) = nk

c     --- Define tangential directions along the boundary

      if ( iface.le.2 ) then

c     --- Faces 1 and 2

         tdir(1) = 2
         tdir(2) = 3
         if ( iface.eq.1 ) then
            iarea(1,1) = 2
            iarea(1,2) = 2
            gammad = 1
         else
            iarea(1,1) = ni
            iarea(1,2) = ni
            gammad = 2
         end if

      else if ( iface.le.4 ) then

c     --- Faces 3 and 4

         tdir(1) = 3
         tdir(2) = 1
         if ( iface.eq.3 ) then
            iarea(2,1) = 2
            iarea(2,2) = 2
            gammad = 1
         else
            iarea(2,1) = nj
            iarea(2,2) = nj
            gammad = 2
         end if

      else

c     --- Faces 5 and 6

         tdir(1) = 1
         tdir(2) = 2
         if ( iface.eq.5 ) then
            iarea(3,1) = 2
            iarea(3,2) = 2
            gammad = 1
         else
            iarea(3,1) = nk
            iarea(3,2) = nk
            gammad = 2
         end if

      end if

c     --- For all points on the boundary do

      do i = iarea(1,1), iarea(1,2)
         do j = iarea(2,1), iarea(2,2)
            do k = iarea(3,1), iarea(3,2)

c           --- Find the type of boundary condition in the point

               mxp(1) = i
               mxp(2) = j
               mxp(3) = k
               btype = ibinfor(2*mxp(tdir(1)),2*mxp(tdir(2)))
               if ( btype.eq.unut .or. btype.eq.uxuy ) then

c              --- velocity is prescribed in point
c                  update right-hand side
c                  set corresponding value in prems equal to zero

                  sum = 0d0
                  do l =1 ,3
                     sum = sum+prems(i,j,k,l,gammad)*
     +                     bndcon(l,2*mxp(tdir(1)),2*mxp(tdir(2)))
                     prems(i,j,k,l,gammad) = 0d0
                  end do
                  rhside(i,j,k,alphad) = rhside(i,j,k,alphad)-sum
               end if
            end do
         end do
      end do
      end
