      subroutine isccrhsd ( ni, nj, nvirtual, rhside, vsum,
     +                      veloc, alphad, intp, iarea, invjac )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 27-08-1999 Duncan: include invjac in distribution
c                                               subroutines
c        version 1.0    date 18-02-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Right-hand side part of newton linearized convection term
c    Classical discretization (conservative)
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, intp, iarea(2,2)
      double precision rhside(*),
     +                 vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 veloc((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual))

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     iarea     i    Area of arrays
c     intp      i    Interpolated values of variable
c     invjac    i    1/sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  kdir, dcell(2)

c     dcell     reserved
c     kdir      General loop variable for 1...ndim loop
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXPY2   Perform a vector*vector update for ISNaS over a
c               2-dimensional region
c     ISAXPY4   Perform a vector*vector update for ISNaS over a
c               2-dimensional region
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    For each velocity component in the integration point do
c          f := f + vsum * u / 2
c
c    The integration points 1 and 2 correspond actually to 2 different points:
c    one with a negative sign and one with a positive sign:
c
c    (-1,0) and (1,0) or (0,-1) and (0,1)
c
c    The right-hand side is updated by a factor
c    update_matrix * velocity / 2
c    The right-hand side vector is situated in the
c    centres of the integration cells
c    The update matrix is positioned in the
c    integration point. So the multiplication must
c    be caried out in the integration points
c    As a consequence both the velocity vactor and
c    the matrix must be shifted to get the correct cell number
c
c    The shift is as follows:
c
c    alpha = 1: Point (-1,0), shift (-1,0)   Point (0,1), shift (0,1)
c    alpha = 2: Point (0,-1), shift (0,-1)   Point (1,0), shift (1,0)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Initialize shifts

      dcell(1) = 0
      dcell(2) = 0

      if ( intp.eq.0 ) then

c     --- Cell centre (0,0), no shift required

         do kdir = 1, 2

c        --- Newton linearization

            call isaxpy4 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                      dcell(1), dcell(2), rhside, 0.5d0,
     +                      vsum(1,kdir), veloc(1,kdir), iarea,
     +                      invjac )

         end do

      else if ( intp.eq.1 ) then

c     --- If V^1 cell then (-1,0) and (1,0)
c         If V^2 cell then (0,-1) and (0,1)

         kdir = alphad

c        --- Newton linearization
c            First left or under point

         dcell(alphad) = -1
         call isaxpy2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                  vsum(1,kdir), iarea, rhside,
     +                  -0.5d0, veloc(1,kdir),
     +                      invjac  )

c        --- Next right or upper point

         dcell(alphad) = 1
         call isaxpy2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                  vsum(1,kdir), iarea, rhside,
     +                  0.5d0, veloc(1,kdir),
     +                      invjac  )

      else

c     --- If V^1 cell then (0,-1) and (0,1)
c         If V^2 cell then (-1,0) and (1,0)

         do kdir = 1, 2

c        --- Newton linearization
c            First left or under point

            dcell(3-alphad) = 0
            call isaxpy2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                     vsum(1,kdir), iarea, rhside,
     +                     -0.5d0, veloc(1,kdir),
     +                      invjac  )

c           --- Next right or upper point

            dcell(3-alphad) = 1
            call isaxpy2 ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                     vsum(1,kdir), iarea, rhside,
     +                     0.5d0, veloc(1,kdir),
     +                      invjac  )
         end do

      end if

      end
