      subroutine iscontg3 ( adirii, cva, ni, nj, nk,
     +                      nvirtual, locdx, alphad )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 13-05-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute integration path coefficients for the velocity gradient
c    three-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad, locdx(3)
      double precision adirii(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3,3),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,*)

c     adirii    o    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, pnttype(3), ishift(3), pnttyp,
     +        iarea(3,2), l, m, l1, m1, valpha(3), betad, gammad
      double precision det, fact(3), cmat(3,3)

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cmat      c_matrix
c     det       Determinant
c     fact      Multiplication factor
c     gammad    Direction index
c     i         General loop variable
c     iarea     Area of arrays
c     ishift    Shift in i-direction
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     l1        General loop variable
c     m         General loop variable
c     m1        General loop variable
c     pnttyp    reserved
c     pnttype   Array containing the point types for the various directions
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The stress tensor in case of the WesBeek discretization is evaluated over
c    6 integration points with respect to a alphad cell
c    The integration points arise by a shift of (-1,0,0), (1,0,0), (0,-1,0),
c    (0,1,0), (0,0,-1) or (0,0,-1) with respect to the cell center
c    In the integration points we need the value of grad(u), where u is the
c    Cartesian velocity vector
c
c    The velocity gradient is expressed in the velocity vector in
c    6 surrounding points of the integration points using the integration path
c    method
c    These equations read:
c
c    grad u (ip) . delta x = u (ip+deltax/2) - u (ip-deltax/2)
c    grad u (ip) . delta y = u (ip+deltay/2) - u (ip-deltay/2)
c    grad u (ip) . delta z = u (ip+deltaz/2) - u (ip-deltaz/2)
c
c    Hence for each velocity component we have 3 equations to express the
c    gradient. Soving this system expresses the gradient in the velocities
c    The co-ordinate differences in the points have already been stored
c    in array cva
c
c    In the integration points on the boundaries we take one-sided
c    differences. In fact this should only be done for certain types of
c    boundary conditions. For example in case of periodical of multiblock
c    boundary conditions this is incorrect. In case of Dirichlet boundary
c    conditions, however, this is the correct approach.
c
c    So one might state that this subroutine is only adapted for Dirichlet
c    boundary conditions.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- First the internal cells
c         Cylcic permutation is stored in array valpha

      valpha(1) = alphad
      valpha(2) = mod(alphad  ,3) + 1
      valpha(3) = mod(alphad+1,3) + 1

c     --- Find the point type number of the integration point
c         Store in pnttyp

      if ( locdx(valpha(1)).ne.0 ) then
         pnttyp = 8
      else if ( locdx(valpha(2)).ne.0 ) then
         if ( valpha(1).eq.1) then
            pnttyp = 5
         else if ( valpha(1).eq.2) then
            pnttyp = 2
         else
            pnttyp = 3
         end if
      else
         if ( valpha(1).eq.1) then
            pnttyp = 3
         else if ( valpha(1).eq.2) then
            pnttyp = 5
         else
            pnttyp = 2
         end if
      end if

c     --- Store the bounds for the loops in iarea
c         iarea(i,1) defines the lower bound in the i-th direction
c         iarea(i,2) defines the upper bound in the i-th direction

      iarea(1,1) = 0
      iarea(1,2) = ni+1
      iarea(2,1) = 0
      iarea(2,2) = nj+1
      iarea(3,1) = 0
      iarea(3,2) = nk+1

      do i = iarea(1,1), iarea(1,2)
         do j = iarea(2,1), iarea(2,2)
            do k = iarea(3,1), iarea(3,2)

c           --- Fill array Cmat with the coordinate differences

               do l = 1, 3

               cmat(1,l) = cva(i,j,k,1,valpha(l),pnttyp)
               cmat(2,l) = cva(i,j,k,2,valpha(l),pnttyp)
               cmat(3,l) = cva(i,j,k,3,valpha(l),pnttyp)

               end do
               det = 1d0/
     +           (cmat(1,1)*(cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +           +cmat(1,2)*(cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +           +cmat(1,3)*(cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))

c              --- Store the inverse of cmat in array adirii

               adirii(i,j,k,1,1) = det*(cmat(2,2)*cmat(3,3)-
     +                                  cmat(2,3)*cmat(3,2))
               adirii(i,j,k,1,2) = det*(cmat(3,2)*cmat(1,3)-
     +                                  cmat(3,3)*cmat(1,2))
               adirii(i,j,k,1,3) = det*(cmat(1,2)*cmat(2,3)-
     +                                  cmat(1,3)*cmat(2,2))

               adirii(i,j,k,2,1) = det*(cmat(2,3)*cmat(3,1)-
     +                                  cmat(2,1)*cmat(3,3))
               adirii(i,j,k,2,2) = det*(cmat(3,3)*cmat(1,1)-
     +                                  cmat(3,1)*cmat(1,3))
               adirii(i,j,k,2,3) = det*(cmat(1,3)*cmat(2,1)-
     +                                  cmat(1,1)*cmat(2,3))

               adirii(i,j,k,3,1) = det*(cmat(2,1)*cmat(3,2)-
     +                                  cmat(2,2)*cmat(3,1))
               adirii(i,j,k,3,2) = det*(cmat(3,1)*cmat(1,2)-
     +                                  cmat(3,2)*cmat(1,1))
               adirii(i,j,k,3,3) = det*(cmat(1,1)*cmat(2,2)-
     +                                  cmat(1,2)*cmat(2,1))
            end do
         end do
      end do

      if ( locdx(alphad).eq.0 ) then

c     --- On the boundaries we use one-sided differences
c         At this moment this is done for all boundaries
c         In case of periodical or multiblock boundary conditions we should not
c         use one-sided differences, hence this subroutine must be adapted in
c         the future
c         The boundary in the alphad direction does not need to be approximated
c         by one-sided differences, hence we jump in the case of
c         integration points in the alphad direction
c         Store the shifts for the one-sided approach in ishift
c         Store the point types of the points for the one-sided differentiation
c         in pnttype
c         betad defines the direction of the integration point with respect
c         to the cell center

         ishift(1) = 0
         ishift(2) = 0
         ishift(3) = 0
         pnttype(1) = pnttyp
         pnttype(2) = pnttyp
         pnttype(3) = pnttyp

c        --- Compute betad

         if ( locdx(1).ne.0 ) then
            betad = 1
         else if ( locdx(2).ne.0 ) then
            betad = 2
         else
            betad = 3
         end if

c        --- For the one sided differences we need the difference in coordinates
c            between the intgration point and the cell center
c            This is 1/2 times the cva value in the cell center
c            Hence we use the point type corresponding to the cell center
c            in the betad direction

         if ( alphad.eq.1 ) then
            pnttype(betad) = 7
         else if ( alphad.eq.2 ) then
            pnttype(betad) = 6
         else
            pnttype(betad) = 4
         end if

c        --- The multiplication factors are stored in fact
c            usually they are 1, but for one-sided differences 1/2

         fact(1) = 1d0
         fact(2) = 1d0
         fact(3) = 1d0
         fact(betad) = 0.5d0

c        --- integration point in betad direction
c            consider the boundaries betad=1 and betad=n_betad+1 separately

         do m = 1, 2
            iarea(1,1) = 0
            iarea(1,2) = ni+1
            iarea(2,1) = 0
            iarea(2,2) = nj+1
            iarea(3,1) = 0
            iarea(3,2) = nk+1
            if ( m.eq.1 ) then

c           --- Begin boundary (i=1, j=1 or k=1)

               iarea(betad,1) = 1
               iarea(betad,2) = 1
               ishift(betad) = 0

            else

c           --- End boundary (ni+1/nj+1/nk+1)

               iarea(betad,1) = iarea(betad,2)
               ishift(betad) = -1

            end if
            do i = iarea(1,1), iarea(1,2)
               do j = iarea(2,1), iarea(2,2)
                  do k = iarea(3,1), iarea(3,2)

c                 --- Fill array C with the coordinate differences

                     do l = 1, 3
                        do m1 = 1, 3
                           cmat(m1,l) = cva(i+ishift(1),j+ishift(2),
     +                                      k+ishift(3),m1,valpha(l),
     +                                      pnttype(valpha(l)))*
     +                                      fact(valpha(l))
                        end do
                     end do
                     det = 1d0/(cmat(1,1)*
     +                     (cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +                     +cmat(1,2)*
     +                     (cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +                     +cmat(1,3)*
     +                     (cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))
                     adirii(i,j,k,1,1) = det*(cmat(2,2)*cmat(3,3)-
     +                                        cmat(2,3)*cmat(3,2))
                     adirii(i,j,k,1,2) = det*(cmat(3,2)*cmat(1,3)-
     +                                        cmat(3,3)*cmat(1,2))
                     adirii(i,j,k,1,3) = det*(cmat(1,2)*cmat(2,3)-
     +                                        cmat(1,3)*cmat(2,2))

                     adirii(i,j,k,2,1) = det*(cmat(2,3)*cmat(3,1)-
     +                                        cmat(2,1)*cmat(3,3))
                     adirii(i,j,k,2,2) = det*(cmat(3,3)*cmat(1,1)-
     +                                        cmat(3,1)*cmat(1,3))
                     adirii(i,j,k,2,3) = det*(cmat(1,3)*cmat(2,1)-
     +                                        cmat(1,1)*cmat(2,3))

                     adirii(i,j,k,3,1) = det*(cmat(2,1)*cmat(3,2)-
     +                                        cmat(2,2)*cmat(3,1))
                     adirii(i,j,k,3,2) = det*(cmat(3,1)*cmat(1,2)-
     +                                        cmat(3,2)*cmat(1,1))
                     adirii(i,j,k,3,3) = det*(cmat(1,1)*cmat(2,2)-
     +                                        cmat(1,2)*cmat(2,1))
                  end do
               end do
            end do
         end do

c        --- Finally we consider the two edges corresponding to the faces
c            that must be approximated one-sided
c            The third direction is called gammad
c            alphad is the direction of the integration cell
c            betad is the direction of the integration point
c            gammad is the other direction

         if ( alphad.eq.1 ) then
            if ( betad.eq.2 ) then
               gammad = 3
            else
               gammad = 2
            end if
         else if ( alphad.eq.2 ) then
            if ( betad.eq.3 ) then
               gammad = 1
            else
               gammad = 3
            end if
         else
            if ( betad.eq.1 ) then
               gammad = 2
            else
               gammad = 1
            end if
         end if
         fact(betad) = 0.5d0
         fact(gammad) = 0.5d0
         pnttype(gammad) = pnttype(betad)

         do m = 1, 2
            iarea(1,1) = 0
            iarea(1,2) = ni+1
            iarea(2,1) = 0
            iarea(2,2) = nj+1
            iarea(3,1) = 0
            iarea(3,2) = nk+1
            if ( m.eq.1 ) then

c           --- Begin boundary (i=1, j=1 or k=1)

               iarea(betad,1) = iarea(betad,2)
               ishift(betad) = -1

            else

c           --- End boundary (ni+1/nj+1/nk+1)

               iarea(betad,1) = 1
               iarea(betad,2) = 1
               ishift(betad) = 0

            end if
            do l1 = 1, 2
               if ( l1.eq.1 ) then

c              --- Begin boundary (i=1, j=1 or k=1)

                  iarea(gammad,1) = iarea(gammad,2)
                  ishift(gammad) = -1

               else

c              --- End boundary (ni+1/nj+1/nk+1)

                  iarea(gammad,1) = 1
                  iarea(gammad,2) = 1
                  ishift(gammad) = 0

               end if
               do i = iarea(1,1), iarea(1,2)
                  do j = iarea(2,1), iarea(2,2)
                     do k = iarea(3,1), iarea(3,2)

c                    --- Fill array C with the coordinate differences

                        do l = 1, 3
                           do m1 = 1, 3
                              cmat(m1,l) = cva(i+ishift(1),j+ishift(2),
     +                                         k+ishift(3),m1,valpha(l),
     +                                         pnttype(valpha(l)))*
     +                                         fact(valpha(l))
                           end do
                        end do
                        det = 1d0/(cmat(1,1)*
     +                        (cmat(2,2)*cmat(3,3)-cmat(2,3)*cmat(3,2))
     +                        +cmat(1,2)*
     +                        (cmat(2,3)*cmat(3,1)-cmat(2,1)*cmat(3,3))
     +                        +cmat(1,3)*
     +                        (cmat(2,1)*cmat(3,2)-cmat(2,2)*cmat(3,1)))
                        adirii(i,j,k,1,1) = det*(cmat(2,2)*cmat(3,3)-
     +                                           cmat(2,3)*cmat(3,2))
                        adirii(i,j,k,1,2) = det*(cmat(3,2)*cmat(1,3)-
     +                                           cmat(3,3)*cmat(1,2))
                        adirii(i,j,k,1,3) = det*(cmat(1,2)*cmat(2,3)-
     +                                           cmat(1,3)*cmat(2,2))

                        adirii(i,j,k,2,1) = det*(cmat(2,3)*cmat(3,1)-
     +                                           cmat(2,1)*cmat(3,3))
                        adirii(i,j,k,2,2) = det*(cmat(3,3)*cmat(1,1)-
     +                                           cmat(3,1)*cmat(1,3))
                        adirii(i,j,k,2,3) = det*(cmat(1,3)*cmat(2,1)-
     +                                           cmat(1,1)*cmat(2,3))

                        adirii(i,j,k,3,1) = det*(cmat(2,1)*cmat(3,2)-
     +                                           cmat(2,2)*cmat(3,1))
                        adirii(i,j,k,3,2) = det*(cmat(3,1)*cmat(1,2)-
     +                                           cmat(3,2)*cmat(1,1))
                        adirii(i,j,k,3,3) = det*(cmat(1,1)*cmat(2,2)-
     +                                           cmat(1,2)*cmat(2,1))
                     end do
                  end do
               end do
            end do
         end do

      end if

      end
