      subroutine isdisdt2 ( alphad, ni, nj, nvirtual, matrix, dtsum,
     +                      mct, iarea, dcell, ichoisdis )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.1    date   02-05-1996 some corrections
c        version 1.0    date   30-10-1995 Subroutine extracted from isdcmadm
c
c
c   copyright (c) 1995-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Distribute diffusion term for tranport equation in R2
c     Both standard distribution as well as monotone distributions are possible
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iarea(2,2), dcell(2), alphad, ichoisdis,
     +        mct(-2:2,-2:2,-2:2)
      double precision matrix(ni+1,nj+1,*),
     +                 dtsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     dtsum     i    Flux derivative contributions to the matrix
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ishift, jshift, ip, jp, betad, minusg, dx(2,2), ofstnr(6)
      double precision faca, facb

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     faca      A factor used in the linear extrapolation formulae
c     facb      A factor used in the linear extrapolation formulae
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     minusg    Indicates if a minus sign has been found (-1) or not (1)
c     ofstnr    Offset diagonal number of matrix element that
c               corresponds to the current point of the interpolation
c               or extrapolation formula
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Consider a transport cell with surrounding cells:
c
c            -------------------------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     -     +     -     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            -------------------------------------
c
c
c     If the integration point is in the 1-direction (locdx(1)=+/-1)
c     then the contribution in the integration point must be added to
c     the correct matrix elements multiplied by a factor.
c
c     Since dtsum contains the partial derivatives the factors are
c     given by the finite difference formula.
c     For example in (1,0) we have:
c
c     dT/dksi_1 = T(2,0) - T(0,0)
c     dT/dksi_2 = (T(2,2) - T(2,0) + T(0,2) - T(0,0) ) / 4
c
c     If ichoisdis=1 we use the scheme proposed by Demirdzic (1982)
c     This scheme, is based on 2-point
c     rather than 4-point interpolation. For example, assuming that we
c     want to discretize the following term
c
c      12 dT                       12
c     K   --  at (i+1/2,j), where K   is a contravariant diffusion tensor
c         dx2
c                       12
c     and suppose that K  (i+1/2,j) > 0, then the following approximation is
c     taken
c
c     dT              1
c     -- |          = - (T      + T         - T        - T       )
c     dx2 (i+1/2,j)   2   (i,j)    (i+1,j+1)   (i,j-1)    (i+1,j)
c
c
c     The interpolation scheme is described in:
c
c     Finite volume discretization of the k-eps turbulence model in
c     general coordinates,
c     Marcel Zijlema,
c     TUD Report 93-90, 1993.
c
c     If ichoisdis=2 we use the scheme proposed by Zijlema (1993)
c     This scheme, is rather
c     one-sided than central difference interpolation. For example,
c     assuming that we want to discretize the following term
c
c      12 dT                       12
c     K   --  at (i+1/2,j), where K   is a contravariant diffusion tensor
c         dx2
c                       12
c     and suppose that K  (i+1/2,j) > 0, then the following approximation is
c     taken
c
c     dT              1
c     -- |          = - (T          + T       )  - T
c     dx2 (i+1/2,j)   2   (i+1,j+1)    (i,j+1)      (i,j)
c
c
c     The interpolation scheme is described in:
c
c     Finite volume discretization of the k-eps turbulence model in
c     general coordinates,
c     Marcel Zijlema,
c     TUD Report 93-90, 1993.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Shift from integration cell to dtsum; i.e. to cell
c         containing the integration point
c         (Rule: for each grid cell left and lower side are included,
c          top and right hand side are in the next cell)

      ishift = dcell(1)
      jshift = dcell(2)

c     --- Compute the two connections for the standard derivatives

      dx(1,1) = 0
      dx(2,1) = 0
      dx(alphad,1) = 1
      betad = 3-alphad
      ofstnr(1) = mct(ishift,jshift,0)
      ofstnr(2) = mct(ishift-dx(1,1),jshift-dx(2,1),0)

c     --- Make a choice between central, monotonous and half monotonous scheme

      if ( ichoisdis.eq.0 ) then

c     --- central scheme
c         Compute the four connections for the cross derivatives

         if ( alphad.eq.1 ) then
            ofstnr(3) = mct(ishift  , 1,0)
            ofstnr(4) = mct(ishift-1, 1,0)
            ofstnr(5) = mct(ishift  ,-1,0)
            ofstnr(6) = mct(ishift-1,-1,0)
         else
            ofstnr(3) = mct( 1,jshift  ,0)
            ofstnr(4) = mct( 1,jshift-1,0)
            ofstnr(5) = mct(-1,jshift  ,0)
            ofstnr(6) = mct(-1,jshift-1,0)
         end if

         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)
               faca = dtsum(ip+ishift,jp+jshift,alphad)
               facb = dtsum(ip+ishift,jp+jshift,betad)*0.25d0
               matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                                   faca
               matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) -
     +                                   faca
               matrix(ip,jp,ofstnr(3)) = matrix(ip,jp,ofstnr(3)) +
     +                                   facb
               matrix(ip,jp,ofstnr(4)) = matrix(ip,jp,ofstnr(4)) +
     +                                   facb
               matrix(ip,jp,ofstnr(5)) = matrix(ip,jp,ofstnr(5)) -
     +                                   facb
               matrix(ip,jp,ofstnr(6)) = matrix(ip,jp,ofstnr(6)) -
     +                                   facb
            end do
         end do

      else if ( ichoisdis.eq.2 ) then

c     --- half monotonous scheme

         dx(1,1) = 0
         dx(2,1) = 0
         dx(1,2) = 0
         dx(2,2) = 0
         dx(1,alphad) = -1
         dx(2,betad) = -1
         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)
               faca = dtsum(ip+ishift,jp+jshift,alphad)
               facb = dtsum(ip+ishift,jp+jshift,betad)*0.5d0
               minusg = sign(1d0,facb)
               facb = abs(facb)
               ofstnr(3) = mct(ishift,jshift,0)
               ofstnr(4) = mct(ishift+dx(1,1),jshift+dx(1,2),0)
               ofstnr(5) = mct(ishift-minusg*dx(2,1),
     +                             jshift-minusg*dx(2,2),0)
               ofstnr(6) = mct(ishift+dx(1,1)+minusg*dx(2,1),
     +                             jshift+dx(1,2)+minusg*dx(2,2),0)
               matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                                   faca
               matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) -
     +                                   faca
               matrix(ip,jp,ofstnr(3)) = matrix(ip,jp,ofstnr(3)) -
     +                                   facb
               matrix(ip,jp,ofstnr(4)) = matrix(ip,jp,ofstnr(4)) +
     +                                   facb
               matrix(ip,jp,ofstnr(5)) = matrix(ip,jp,ofstnr(5)) +
     +                                   facb
               matrix(ip,jp,ofstnr(6)) = matrix(ip,jp,ofstnr(6)) -
     +                                   facb
            end do
         end do

      else

c     --- monotonous scheme

         dx(1,1) = 0
         dx(2,1) = 0
         dx(1,2) = 0
         dx(2,2) = 0
         dx(1,alphad) = -1
         dx(2,betad) = -1
         do jp = iarea(1,2),iarea(2,2)
            do ip = iarea(1,1),iarea(2,1)
               faca = dtsum(ip+ishift,jp+jshift,alphad)
               facb = dtsum(ip+ishift,jp+jshift,betad)
               minusg = sign(1d0,facb)
               facb = abs(facb)
               ofstnr(3) = mct(ishift+dx(1,1),jshift+dx(1,2),0)
               ofstnr(4) = mct(ishift-minusg*dx(2,1),
     +                             jshift-minusg*dx(2,2),0)
               ofstnr(5) = mct(ishift+dx(1,1)-minusg*dx(2,1),
     +                             jshift+dx(1,2)-minusg*dx(2,2),0)
               matrix(ip,jp,ofstnr(1)) = matrix(ip,jp,ofstnr(1)) +
     +                                   faca
               matrix(ip,jp,ofstnr(2)) = matrix(ip,jp,ofstnr(2)) -
     +                                   faca
               matrix(ip,jp,ofstnr(3)) = matrix(ip,jp,ofstnr(3)) -
     +                                   facb
               matrix(ip,jp,ofstnr(4)) = matrix(ip,jp,ofstnr(4)) +
     +                                   0.5d0*facb
               matrix(ip,jp,ofstnr(5)) = matrix(ip,jp,ofstnr(5)) +
     +                                   0.5d0*facb
            end do
         end do

      end if

      end
