      subroutine isdoub2d( cntdef, pderiv, veloc, cva, invjac, jcta,
     +                     jctg, chsym, ni, nj, nvirtual, idiscret,
     +                     pnttyp, ksign                               )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.3    date   18-05-1998 Charles Bilinear discretization
c        version 1.2    date   21-08-1996 Marcel: solut --> veloc
c        version 1.1    date   22-05-1996 Marcel: new variable ksign
c        version 1.0    date   08-12-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the double contraction of rate-of-deformation tensor or
c     rotation tensor.
c     If standard discretization, then
c                    ab                    ab
c                   S  S          or      W  W
c                       ab                    ab
c               ab   1   bc  a     ac  b               1
c     where    S   = -( g   U   + g   U  )  and  S   = -(U    + U   )
c                    2       ,c        ,c         ab   2  a,b    b,a
c               ab   1   bc  a     ac  b               1
c     and      W   = -( g   U   - g   U  )  and  W   = -(U    - U   )
c                    2       ,c        ,c         ab   2  a,b    b,a
c     Otherwise, compute the double contraction of rate-of-deformation
c     tensor  s  s    or rotation tensor w  w   using WESBEEK method
c              ab ab                      ab ab
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, idiscret, pnttyp
      double precision ksign
      double precision cntdef(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 pderiv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1:2,1:2),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:2,1:3)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,3),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3)

c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cntdef    o    Resulted double contraction of deformation rate tensor
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ksign     i    Help parameter to define 1 or -1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pderiv    i    Partial derivative of flux in point pnttyp
c     pnttyp    i    reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, alphad, alphal, betad, betal, deltal, i, j, idir,
     +        dx(1:2), gammal, vcellc, istwo(1:2,1:2)
      double precision gac, gbc, contrst, covrst, crst, u(1:2,1:2,1:2),
     +                 dudx(1:2,1:2)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     alphal    Loop index
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     betal     Loop index
c     contrst   Contravariant rate of strain tensor
c     covrst    Covariant rate of strain tensor
c     crst      Cartesian rate of strain tensor
c     deltal    Loop index
c     dudx      Partial derivative of Cartesian velocities in Cartesian
c               directions
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     gac       Covariant metric tensor
c     gammal    Loop index
c     gbc       Covariant metric tensor
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     u         Cartesian velocities
c     vcellc    Point type of integration cell centre for a given
c               variable type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( idiscret.eq.1 ) then

c     --- standard discretization

        do jp = 1, nj
          do ip = 1, ni

            do alphad = 1, 2
              do betad = 1, 2

                contrst = 0d0
                covrst  = 0d0

                do gammal = 1, 2

                  gac = 0d0
                  gbc = 0d0

                  do i = 1, 2
                    gac = gac + cva(ip,jp,i,alphad,pnttyp) *
     +                          cva(ip,jp,i,gammal,pnttyp)
                    gbc = gbc + cva(ip,jp,i,betad,pnttyp)  *
     +                          cva(ip,jp,i,gammal,pnttyp)
                  end do

                  contrst = contrst + 5d-1*invjac(ip,jp,pnttyp) *
     +                      invjac(ip,jp,pnttyp) *
     +                      ( jctg(ip,jp,istwo(betad,gammal)) *
     +                        pderiv(ip,jp,alphad,gammal) +
     +                        ksign*jctg(ip,jp,istwo(alphad,gammal)) *
     +                        pderiv(ip,jp,betad,gammal) )

                  covrst = covrst + 5d-1*invjac(ip,jp,pnttyp) *
     +                     ( gac * pderiv(ip,jp,gammal,betad) +
     +                       ksign*gbc * pderiv(ip,jp,gammal,alphad) )

                  do deltal = 1, 2

                    contrst = contrst + 5d-1*invjac(ip,jp,pnttyp) *
     +                  invjac(ip,jp,pnttyp) *
     +                  ( jctg(ip,jp,istwo(betad,gammal)) *
     +                    ( chsym(ip,jp,alphad,istwo(deltal,gammal))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(gammal,deltal))*
     +                      veloc(ip,jp,alphad,3) ) +
     +                    ksign*jctg(ip,jp,istwo(alphad,gammal)) *
     +                    ( chsym(ip,jp,betad,istwo(deltal,gammal))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(gammal,deltal))*
     +                      veloc(ip,jp,betad,3) ) )

                    covrst = covrst + 5d-1*invjac(ip,jp,pnttyp) *
     +                  ( gac *
     +                    ( chsym(ip,jp,gammal,istwo(deltal,betad))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(betad,deltal))*
     +                      veloc(ip,jp,gammal,3) ) +
     +                    ksign*gbc *
     +                    ( chsym(ip,jp,gammal,istwo(deltal,alphad))*
     +                      veloc(ip,jp,deltal,3) -
     +                      chsym(ip,jp,deltal,istwo(alphad,deltal))*
     +                      veloc(ip,jp,gammal,3) ) )

                  end do

                end do

                cntdef(ip,jp) = cntdef(ip,jp) + contrst*covrst

              end do
            end do

          end do
        end do

      else if ( idiscret.eq.2.or.idiscret.eq.4 ) then

c     --- WesBeek discretization

        do jp = 1, nj
          do ip = 1, ni

            do idir = 1, 2

c           --- Set vcellc
c               (translate from variable type to corresponding
c               integration cell centre point type)

              vcellc = 4 - 2**(idir-1)

              do i = 1, 2

                dx(1) = 0
                dx(2) = 0
                dx(idir) = i - 1

                do betal = 1, 2

                  u(idir,i,betal) = 0d0

                  do j = 1, 2
                    u(idir,i,betal) = u(idir,i,betal) +
     +                                cva(ip+dx(1),jp+dx(2),betal,j,
     +                                    vcellc) *
     +                                veloc(ip+dx(1),jp+dx(2),j,idir)
                  end do

                  u(idir,i,betal) = u(idir,i,betal)*
     +                              invjac(ip+dx(1),jp+dx(2),vcellc)

                end do
              end do
            end do

            do alphal = 1, 2
              do betal = 1, 2

                dudx(alphal,betal) = 0d0

                do j = 1, 2

                  dudx(alphal,betal) = dudx(alphal,betal) +
     +                                 jcta(ip,jp,betal,j) *
     +                                 invjac(ip,jp,pnttyp) *
     +                                 ( u(j,2,alphal) - u(j,1,alphal) )

                end do

              end do
            end do

            cntdef(ip,jp) = 0d0

            do alphal = 1, 2
              do betal = 1, 2

                crst = 5d-1*(dudx(alphal,betal) +
     +                       ksign*dudx(betal,alphal))

                cntdef(ip,jp) = cntdef(ip,jp) + crst*crst

              end do
            end do

          end do
        end do

      end if

      end
