      subroutine isglbash3 ( iface, alphad, ishiftlc, ishiftgl,
     +                       pnttyput, ibounsh, iextrash )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 20-05-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute the shift in global numbering given the local shift and the
c    cell center type. Also the new point type number is detected.
c    This subroutine is meant for faces at the boundary.
c    Not only the global shift is detected but also the shift in position
c    in the boundary array.
c    With respect to the local shift, each cell has a length of 2 in each
c    direction
c    3d version
c **********************************************************************
c
c                       KEYWORDS
c
c    point_type
c    shift
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iface, alphad, ishiftlc(3), ishiftgl(3), pnttyput,
     +        ibounsh(2), iextrash(2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibounsh   o    Array containing the shift in global coordinates with
c                    respect to the boundary cells
c                    For faces 1 and 2 ibounsh refers to j (1) and k (2)
c                    For faces 3 and 4 ibounsh refers to k (1) and i (2)
c                    For faces 5 and 6 ibounsh refers to i (1) and j (2)
c                    Together with iextrash the position in the boundary cell
c                    may be determined
c     iextrash  o    Array containing the extra shift in global coordinates with
c                    respect to the boundary cells
c                    Together with ibounsh the position in the boundary cell
c                    may be determined as follows
c                    For faces 1 and 2:  (2*(j+ibounsh(1))-1+iextrash(1),
c                                         2*(k+ibounsh(2))-1+iextrash(2))
c                    For faces 3 and 4:  (2*(k+ibounsh(1))-1+iextrash(1),
c                                         2*(i+ibounsh(2))-1+iextrash(2))
c                    For faces 5 and 6:  (2*(i+ibounsh(1))-1+iextrash(1),
c                                         2*(j+ibounsh(2))-1+iextrash(2))
c     iface     i    Face number
c     ishiftgl  o    Array containing the shift in global coordinates
c     ishiftlc  i    Array containing the shift in local coordinates
c     pnttyput  o    Point type of point where the shift goes to
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer pnttypin, ishiftpn(3,8), locshift(3), k

c     ishiftpn  Array of size ndim x 2^ndim containing the shifts of
c               points in the cell with respect to the left under point
c               ishiftpn(i,pnttyp) contains the shift in i-direction for
c               a point with point type pnttyp
c     k         General loop variable
c     locshift  Array containing the local shifts
c     pnttypin  Point type of point from which shift must be taken
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    From the point type we can detect a local shift with respect to
c    the "left under" point of a cell (point type 1)
c    This local shift is stored in array ishiftpn
c    The actual local shift is defined by shift + ishift(.,pnttypin)
c    This is stored in locshift
c    The global shift satisfies glob_shift = locshift/2, where the
c    FORTRAN integer division is applied
c    To compute the new point type we proceed as follows:
c       locshift := locshift-2*glob_shift
c       pnttype = 1+locshift(1)*1+locshift(2)*2+locshift(3)*4
c
c    Once the shift and output point type are known the position in the
c    boundary arrays can be detected depending on the face sequence number
c
c    The position in the boundary arrays is detected as follows:
c
c    (ipos1,ipos2)
c
c    For face 1, 2 ipos1 refers to the j direction and ipos2 to the k direction
c    For face 3, 4 ipos1 refers to the k direction and ipos2 to the i direction
c    For face 5, 6 ipos1 refers to the i direction and ipos2 to the j direction
c
c    For faces 1 and 2 we find the position in the boundary array by:
c
c    ipos1 = 2*(j+ibounsh(1))-1+iextrash(1)
c    ipos2 = 2*(k+ibounsh(2))-1+iextrash(2)
c
c    For the other faces j and k must be replaced by the appropriate variables
c
c    The global shift is copied in ibounsh
c
c    The extra shift is detected as follows:
c
c    If face 1 or 2 then iextrash(1) = 1 if pnttype_in = 7 and pnttype_out=3,7
c                        iextrash(2) = 1 if pnttype_in = 4 and pnttype_out=5,7
c    If face 3 or 4 then iextrash(1) = 1 if pnttype_in = 4 and pnttype_out=5,6
c                        iextrash(2) = 1 if pnttype_in = 6 and pnttype_out=2,6
c    If face 5 or 6 then iextrash(1) = 1 if pnttype_in = 6 and pnttype_out=2,4
c                        iextrash(2) = 1 if pnttype_in = 7 and pnttype_out=3,4
c    Otherwise iextrash(.) = 0
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data ishiftpn / 0, 0, 0,
     +                1, 0, 0,
     +                0, 1, 0,
     +                1, 1, 0,
     +                0, 0, 1,
     +                1, 0, 1,
     +                0, 1, 1,
     +                1, 1, 1 /
c ======================================================================
c
c     --- Compute pnttypin as function of alphad

      if ( alphad.eq.1 ) then
         pnttypin = 7
      else if ( alphad.eq.2 ) then
         pnttypin = 6
      else
         pnttypin = 4
      end if

c     --- Compute the actual local shift

      locshift(1) = ishiftlc(1) + ishiftpn(1,pnttypin)
      locshift(2) = ishiftlc(2) + ishiftpn(2,pnttypin)
      locshift(3) = ishiftlc(3) + ishiftpn(3,pnttypin)

c     --- Compute the global shift

      do k = 1, 3
         if ( locshift(k).gt.0 ) then
            ishiftgl(k) = locshift(k)/2
         else if ( locshift(k).eq.0 ) then
            ishiftgl(k) = 0
         else
            ishiftgl(k) = (locshift(k)-1)/2
         end if
      end do

c     --- Update the actual local shift

      locshift(1) = locshift(1) - ishiftgl(1)*2
      locshift(2) = locshift(2) - ishiftgl(2)*2
      locshift(3) = locshift(3) - ishiftgl(3)*2

c     --- Compute new point type

      pnttyput = 1+locshift(1)+locshift(2)*2+locshift(3)*4

c     --- Copy ishiftgl into ibounsh and fill iextrash as function of iface

      iextrash(1) = 0
      iextrash(2) = 0
      if ( iface.le.2 ) then

c     --- iface = 1,2: j,k

         ibounsh(1) = ishiftgl(2)
         ibounsh(2) = ishiftgl(3)
         if ( pnttypin.eq.4 .and. pnttyput.eq.3 .or. pnttyput.eq.7 )
     +      iextrash(1) = 1
         if ( pnttypin.eq.6 .and. pnttyput.eq.5 .or. pnttyput.eq.7 )
     +      iextrash(2) = 1

      else if ( iface.le.4 ) then

c     --- iface = 3,4: k,i

         ibounsh(1) = ishiftgl(3)
         ibounsh(2) = ishiftgl(1)
         if ( pnttypin.eq.7 .and. pnttyput.eq.5 .or. pnttyput.eq.6 )
     +      iextrash(1) = 1
         if ( pnttypin.eq.4 .and. pnttyput.eq.2 .or. pnttyput.eq.6 )
     +      iextrash(2) = 1

      else

c     --- iface = 5,6: i,j

         ibounsh(1) = ishiftgl(1)
         ibounsh(2) = ishiftgl(2)
         if ( pnttypin.eq.6 .and. pnttyput.eq.2 .or. pnttyput.eq.4 )
     +      iextrash(1) = 1
         if ( pnttypin.eq.7 .and. pnttyput.eq.3 .or. pnttyput.eq.4 )
     +      iextrash(2) = 1

      end if

      end
