      subroutine ismomcw3 ( ni, nj, nk, nvirtual, densrho, adir,
     +                      cva, vsum, alphad, intpnr, dcell )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 25-03-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the coefficient in the integration point
c     within a flux cell for the convection term in R3 (WesBeek)
c     Restriction: at this moment only Picard iteration and central
c                  differences have been programmed
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad, intpnr, dcell(3)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,*),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,*),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     intpnr    i    Integration point under consideration
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, ish, jsh, ksh, idir, iupp,
     +        jupp, kupp
      double precision fact

c     fact      Multiplication factor
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ish       Shift in i-direction
c     iupp      Upper i-index of subface
c     j         General loop variable
c     jsh       Shift in j-direction
c     jupp      Upper j-index of subface
c     k         General loop variable
c     ksh       Shift in k-direction
c     kupp      Upper k-index of subface
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c    The convective terms in case of the WesBeek discretization are given by:
c
c              |i+1/2,j,k                |i,j+1/2,k                |i,j,k+1/2
c    rho V^1 u |           +   rho V^2 u |           +   rho V^3 u |
c              |i-1/2,j,k                |i,j-1/2,k                |i,j,k-1/2
c
c     with u = ( a_(1) V^1 +a_(2) V^2 +a_(2) V^2 ) / sqrt(g)
c     u is a vector
c
c     This term must be multiplied by the vector a^(alpha) in (i,j,k)
c
c     In case of Picard V is taken at the old time level
c
c     The term rho V^beta/sqrt(g) has been computed in the integration point
c     and stored in array densrho
c     In this subroutine a^(alpha) . a_(gamma) rho V^beta/sqrt(g) is
c     computed and stored in array vsum(gamma)
c
c     Hence we compute:
c
c     for gamma := 1 (1) 3 do
c        for all i,j,k do
c           vsum(i+di,j+dj,k+dk,gamma) =
c                a^(alpha)(i,j,k) . a_(gamma)W (i+di,j+dj,k+dk)
c     with W = rho V^beta/sqrt(g)
c     and (di,dj,dk) defines the shift with respect to the global cell
c     numbering
c     The shift is stored in array dcell
c     hence vsum is stored in the integration point
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( mod(intpnr,2).eq.0 ) then

c     --- Points 2, 4, 6, i.e. negative integration point

         fact =-1d0

      else

c     --- Points 3, 5, 7, i.e. positive integration point

         fact = 1d0

      end if
      iupp = ni
      jupp = nj
      kupp = nk
      if ( alphad.eq.1 ) then
         iupp = ni+1
      else if ( alphad.eq.2 ) then
         jupp = nj+1
      else
         kupp = nk+1
      end if
      ish = dcell(1)
      jsh = dcell(2)
      ksh = dcell(3)

c     --- Loop over three velocity components

      do idir = 1, 3

         do i = 1, iupp
            do j = 1, jupp
               do k = 1, kupp
                  vsum(i+ish,j+jsh,k+ksh,idir) =
     +               fact*densrho(i+ish,j+jsh,k+ksh)*
     +              (adir(i,j,k,1)*cva(i+ish,j+jsh,k+ksh,1,idir)+
     +               adir(i,j,k,2)*cva(i+ish,j+jsh,k+ksh,2,idir)+
     +               adir(i,j,k,3)*cva(i+ish,j+jsh,k+ksh,3,idir))
               end do
            end do
         end do

      end do
      end
