      subroutine ismomrh3 ( f, rho, coor, xcoor, gmat, gmctinv, rhside,
     +                      work, ni, nj, nk, nvirtual )
c ======================================================================
c
c     programmer  Guus Segal
c     version     1.0   date  17-07-1996
c
c   copyright (c) 1996-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine to build the right-hand side source term in R2
c     for one step in the solution process for the momentum equations.
c     rho    * [ Gx     f1     +  Gy     f2     +  Gz     f3 ]
c       (0,0)      (0,0)  (0,0)     (0,0)  (0,0)     (0,0)  (0,0)
c **********************************************************************
c
c                       KEYWORDS
c
c     force_righthandside
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, gmctinv(0:3,3,10)
      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 xcoor(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3),
     +                 work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,5),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 rho(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual),
     +                 f(1-nvirtual:ni+1+nvirtual,
     +                   1-nvirtual:nj+1+nvirtual,
     +                   1-nvirtual:nk+1+nvirtual,3)

c     coor      i    reserved
c     f         i    Array containing the force term
c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rho       i    Estimate of the asymptotic convergence rate
c     rhside   i/o   reserved
c     work           Work array
c     xcoor          Work array in which the co-ordinates in the centroids
c                    are stored
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer idim, i, ishift, j, jshift, k, kshift, l
      double precision f1int, f2int, f3int, rhoint

c     f1int     Value of f_1 in integration point
c     f2int     Value of f_2 in integration point
c     f3int     Value of f_3 in integration point
c     i         General loop variable
c     idim      Loop variable used for dimension
c     ishift    Shift in i-direction
c     j         General loop variable
c     jshift    Shift in j-direction
c     k         General loop variable
c     kshift    Shift in k-direction
c     l         General loop variable
c     rhoint    Value of rho in integration point
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    evaluate x in cell centres and store in xcoor(.,1)
c    evaluate y in cell centres and store in xcoor(.,2)
c    evaluate z in cell centres and store in xcoor(.,3)
c    for idim := 1, 3 do
c       Compute Gx in integration centre and store in work(.,1)
c       Compute Gy in integration centre and store in work(.,2)
c       Compute Gz in integration centre and store in work(.,3)
c       evaluate rho in integration centre
c       evaluate f1  in integration centre
c       evaluate f2  in integration centre
c       evaluate f3  in integration centre
c       Update right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- evaluate x, y and z in cell centres and store in xcoor(.,1)

      do idim = 1, 3
         do i = 0, ni+1
            do j = 0, nj+1
               do k = 0, nk+1
                  xcoor(i,j,k,idim) = 0.125d0*(coor(i,j,k,idim)+
     +                  coor(i+1,j,k,idim)+coor(i,j+1,k,idim)+
     +                  coor(i+1,j+1,k,idim)+coor(i,j,k+1,idim)+
     +                  coor(i+1,j,k+1,idim)+coor(i,j+1,k+1,idim)+
     +                  coor(i+1,j+1,k+1,idim))
               end do
            end do
         end do
      end do

c     --- First evaluation and updating in 1-direction
c         Compute Gx, Gy and Gz, store in work

      call issetcon ( work,
     +    (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)*3, 0d0 )
      do l = 1, 10

         ishift = gmctinv(1,1,l)
         jshift = gmctinv(2,1,l)
         kshift = gmctinv(3,1,l)

         do k = 1, nk
            do j = 1, nj
               do i = 1, ni+1
                  work(i,j,k,1) = work(i,j,k,1) + gmat(i,j,k,1,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,1)
                  work(i,j,k,2) = work(i,j,k,2) + gmat(i,j,k,1,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,2)
                  work(i,j,k,3) = work(i,j,k,3) + gmat(i,j,k,1,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,3)
               end do
            end do
         end do

      end do

c     --- evaluate rho, f1, f2 and f3 in integration centre

      do i = 1, ni+1
         do j = 1, nj
            do k = 1, nk
               rhoint = 0.5d0*(rho(i,j,k)+rho(i-1,j,k))
               f1int = 0.5d0*(f(i,j,k,1)+f(i-1,j,k,1))
               f2int = 0.5d0*(f(i,j,k,2)+f(i-1,j,k,2))
               f3int = 0.5d0*(f(i,j,k,3)+f(i-1,j,k,3))
               rhside(i,j,k,1) = rhside(i,j,k,1)+rhoint*
     +                         (work(i,j,k,1)*f1int+work(i,j,k,2)*f2int+
     +                          work(i,j,k,3)*f3int)
            end do
         end do
      end do

c     --- Next evaluation and updating in 2-direction
c         Compute Gx, Gy and Gz, store in work

      call issetcon ( work,
     +    (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)*3, 0d0 )
      do l = 1, 10

         ishift = gmctinv(1,2,l)
         jshift = gmctinv(2,2,l)
         kshift = gmctinv(3,2,l)

         do i = 1, ni
            do k = 1, nk
               do j = 1, nj+1
                  work(i,j,k,1) = work(i,j,k,1) + gmat(i,j,k,2,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,1)
                  work(i,j,k,2) = work(i,j,k,2) + gmat(i,j,k,2,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,2)
                  work(i,j,k,3) = work(i,j,k,3) + gmat(i,j,k,2,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,3)
               end do
            end do
         end do

      end do

c     --- evaluate rho, f1, f2 and f3 in integration centre

      do i = 1, ni
         do j = 1, nj+1
            do k = 1, nk
               rhoint = 0.5d0*(rho(i,j,k)+rho(i,j-1,k))
               f1int = 0.5d0*(f(i,j,k,1)+f(i,j-1,k,1))
               f2int = 0.5d0*(f(i,j,k,2)+f(i,j-1,k,2))
               f3int = 0.5d0*(f(i,j,k,3)+f(i,j-1,k,3))
               rhside(i,j,k,2) = rhside(i,j,k,2)+rhoint*
     +                         (work(i,j,k,1)*f1int+work(i,j,k,2)*f2int+
     +                          work(i,j,k,3)*f3int)
            end do
         end do
      end do

c     --- Next evaluation and updating in 3-direction
c         Compute Gx, Gy and Gz, store in work

      call issetcon ( work,
     +    (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)*3, 0d0 )
      do l = 1, 10

         ishift = gmctinv(1,3,l)
         jshift = gmctinv(2,3,l)
         kshift = gmctinv(3,3,l)

         do j = 1, nj
            do i = 1, ni
               do k = 1, nk+1
                  work(i,j,k,1) = work(i,j,k,1) + gmat(i,j,k,3,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,1)
                  work(i,j,k,2) = work(i,j,k,2) + gmat(i,j,k,3,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,2)
                  work(i,j,k,3) = work(i,j,k,3) + gmat(i,j,k,3,l) *
     +                            xcoor(i+ishift,j+jshift,k+kshift,3)
               end do
            end do
         end do

      end do

c     --- evaluate rho, f1, f2 and f3 in integration centre

      do i = 1, ni
         do j = 1, nj
            do k = 1, nk+1
               rhoint = 0.5d0*(rho(i,j,k)+rho(i,j,k-1))
               f1int = 0.5d0*(f(i,j,k,1)+f(i,j,k-1,1))
               f2int = 0.5d0*(f(i,j,k,2)+f(i,j,k-1,2))
               f3int = 0.5d0*(f(i,j,k,3)+f(i,j,k-1,3))
               rhside(i,j,k,3) = rhside(i,j,k,3)+rhoint*
     +                         (work(i,j,k,1)*f1int+work(i,j,k,2)*f2int+
     +                          work(i,j,k,3)*f3int)
            end do
         end do
      end do

      end
