      subroutine ismomst2 ( ni, nj, nvirtual, mu, jctg, chsym, vsum,
     +                      dvsum, alphad, locdx, iparea, nconct, jac,
     +                      reduced, mequat )
c ======================================================================
c
c        programmer     Ronald van Nooyen
c        version 5.0    date 08-10-1997 Hester: Extra param mequat
c        version 4.0    date 12-03-1997 Guus: Extra param reduced
c        version 3.1    date 30-01-1997 Hester: Extra param. jac for
c                                       compressibility
c        version 3.0    date 02-10-1996 Guus: Extra parameter nconct
c        version 2.0    date 30-10-1995 Guus: remove nzdvsum
c        version 1.0    date 18-04-1995 Subroutine extracted from ismms2
c
c   copyright (c) 1993-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     determines the coefficients of the flux components and the partial
c     derivatives of the flux components in a given integration point
c     within a flux cell for the stress terms
c     Two-dimensional classical 13-point ISNaS discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    stress
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2), alphad, iparea(2,2), nconct,
     +        mequat
      double precision mu(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,3),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 dvsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,*)
      logical reduced

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     dvsum     o    Flux derivative contributions to the matrix
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jac       i    sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mu        i    The laminar viscosity
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     reduced   i    If true the reduced stress tensor corresponding
c                    to a constant viscosity and incompressible flow is used
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, betad, gammad, i, j, iptensor1, gammalow,
     +        iptensor2, iptensor3, iptensor4, istwo(2,2), gammaupp
      double precision sign, facs
      logical nzdvsum(2,2)

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     facs      Multiplication factor
c     gammad    Direction index
c     gammalow  Lower bound of gamma
c     gammaupp  Upper bound of gamma
c     i         General loop variable
c     ip        Local counting variable i-direction
c     iptensor1 Tensor index
c     iptensor2 Tensor index
c     iptensor3 Tensor index
c     iptensor4 Tensor index
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     nzdvsum   Indicates whether or not a partial derivative in a
c               certain direction occurs in the stencil.
c               For example the derivative dU2/dksi1 in the point (0,1)
c               or the V1-stencil of the 13-point molecule does not exist
c               nzdvsum should therefore be false in that point
c     sign      sign for multiplication (1 or -1)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       Relevant formula
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j              a2 |i,j+1/2            |  a  |   cb |
c   sqrt(g)  T   |         + sqrt(g)  T   |        +  sqrt(g) |     |  T   |
c                |i-1/2,j                 |i,j-1/2            | b c |      |i,j
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c
c   If the incompressibility condition (nconct=13) is substituted then:
c
c    12         11 2      22 1
c  -T   = mu ( g  U    + g  U   )
c                  ,1        ,2
c
c   otherwise (nconct=17):
c
c    12         11 2      22 1          12    1     2
c  -T   = mu ( g  U    + g  U   ) + mu g   ( U   + U   )
c                  ,1        ,2               ,1    ,2
c
c
c                    a                  a        b     |  a   |  c
c    The derivative U   is equal to  d U  / d Ksi    + |      | U
c                    ,b                                |b   c |
c
c                                            a       a        b
c    In this subroutine the coefficients of U and d U  / d Ksi and not of
c    V^a and dV^a/dKsi^b are computed because:
c
c    d U^a / d Ksi^b = d (V^a/sqrt(g)) / d Ksi^b =
c
c                      1/sqrt(g)d d V^a / d Ksi^b + V^a d (1/sqrt(g)) / d Ksi^b
c
c    Hence sqrt(g) must be evaluated in the points where V^a is evaluated,
c    i.e. in the points where the unknowns are positioned.
c    This means that the distribution subroutine takes care of
c    the division by sqrt(g)
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- In first instance we consider all coefficients before the
c         velocity derivative terms DVa/Dksib

      if ( abs(locdx(1))+abs(locdx(2)).eq.1 ) then

c     --- Integration points (0,1), (0,-1), (1,0) and (-1,0)

         if ( abs(locdx(1)).ne.0 ) betad = 1
         if ( abs(locdx(2)).ne.0 ) betad = 2
         sign = -locdx(betad)
         if ( .not. reduced ) then
            facs = 2d0 * sign
         else
            facs = sign
         end if

         if ( alphad.eq.betad ) then

c        --- alphad = betad

            do gammad = 1, 2

               nzdvsum(betad,gammad) = .true.
               iptensor1 = istwo(gammad,alphad)

               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     dvsum(ip,jp,betad,gammad) = facs*
     +                    mu(ip,jp)*jctg(ip,jp,iptensor1)
                  end do
               end do

            end do

            if ( nconct.eq.17 .and. mequat.eq.0 ) then

c           --- compressible part t11 and t22 stress term:
c               -2/3*mu*div(U)

               nzdvsum(1,1) = .true.
               nzdvsum(2,2) = .true.

               facs = 2d0/3d0

               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)

                     dvsum(ip,jp,1,1)=dvsum(ip,jp,1,1)-sign*facs*
     +                     mu(ip,jp)*jac(ip,jp,alphad)
                     dvsum(ip,jp,2,2)=dvsum(ip,jp,2,2)-sign*facs*
     +                     mu(ip,jp)*jac(ip,jp,alphad)

                  end do
               end do

            end if

         else

c        --- alphad # betad

            if ( reduced ) then
               gammalow = betad
               gammaupp = betad
            else
               gammalow = 1
               gammaupp = 2
            end if
            do gammad = gammalow, gammaupp

               j = 3 - gammad
               nzdvsum(j,gammad) = .true.
               iptensor1 = istwo(gammad,gammad)

               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     dvsum(ip,jp,j,gammad) = sign*
     +                    mu(ip,jp)*jctg(ip,jp,iptensor1)
                  end do
               end do

            end do
            if ( nconct.eq.17 ) then

c           --- Incompressibility is not substituted

               nzdvsum(1,1) = .true.
               nzdvsum(2,2) = .true.

               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     dvsum(ip,jp,1,1) = sign*mu(ip,jp)*jctg(ip,jp,2)
                     dvsum(ip,jp,2,2) = dvsum(ip,jp,1,1)
                  end do
               end do

            end if

         end if

      else if ( locdx(1).eq.0 .and. locdx(2).eq.0 ) then

c     --- Centroid point (0,0)
c         This concerns the terms involving the Christoffel symbols
c         First we consider the case gamma = beta (b=c)

         do gammad = 1, 2

            nzdvsum(gammad,gammad) = .true.
            iptensor1 = istwo(gammad,gammad)

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  dvsum(ip,jp,gammad,gammad) =-2d0*mu(ip,jp)*
     +                 chsym(ip,jp,alphad,iptensor1)*
     +                 jctg(ip,jp,iptensor1)
               end do
            end do

         end do

         if ( nconct.eq.17 .and. mequat.eq.1 ) then

c        --- compressible part christoffel*t11 and t22 stress term:
c            -2/3*mu*div(U)

            nzdvsum(1,1) = .true.
            nzdvsum(2,2) = .true.

            facs = 2d0/3d0

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)

                  dvsum(ip,jp,1,1)=dvsum(ip,jp,1,1)+facs*mu(ip,jp)*
     +                  jac(ip,jp,4)*( chsym(ip,jp,alphad,1) +
     +                  chsym(ip,jp,alphad,3) )
                  dvsum(ip,jp,2,2)=dvsum(ip,jp,2,2)+facs*mu(ip,jp)*
     +                  jac(ip,jp,4)*( chsym(ip,jp,alphad,1) +
     +                  chsym(ip,jp,alphad,3) )

               end do
            end do

         end if

c        --- Next we consider gamma#beta (b#c)

         do gammad = 1, 2

            j = 3 - gammad
            nzdvsum(j,gammad) = .true.
            iptensor1 = istwo(j,1)
            iptensor2 = istwo(gammad,1)
            iptensor3 = istwo(j,2)
            iptensor4 = istwo(gammad,2)

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  dvsum(ip,jp,j,gammad) =
     +                 - 2d0*mu(ip,jp)*
     +               ( chsym(ip,jp,alphad,iptensor1)*
     +                 jctg(ip,jp,iptensor2)+
     +                 chsym(ip,jp,alphad,iptensor3)*
     +                 jctg(ip,jp,iptensor4) )
               end do
            end do
         end do

         if ( nconct.eq.17 ) then

c        --- Incompressibility is not substituted

            nzdvsum(1,1) = .true.
            nzdvsum(2,2) = .true.

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  dvsum(ip,jp,1,1) = -mu(ip,jp)*jctg(ip,jp,2)*
     +                                chsym(ip,jp,alphad,2)
                  dvsum(ip,jp,2,2) = dvsum(ip,jp,1,1)
               end do
            end do

         end if

      end if

c     --- Finally all coefficients before Ua

      do i = 1, 2

         do j = 1, 2

            if ( nzdvsum(i,j) ) then

c           --- Point is in stencil

               iptensor1 = istwo(j,1)
               iptensor2 = istwo(j,2)

               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     vsum(ip,jp,1) = vsum(ip,jp,1)+
     +                    dvsum(ip,jp,i,j)*
     +                    chsym(ip,jp,i,iptensor1)
                     vsum(ip,jp,2) = vsum(ip,jp,2)+
     +                    dvsum(ip,jp,i,j)*
     +                    chsym(ip,jp,i,iptensor2)
                 end do
              end do

            end if

         end do
      end do

      end
