      subroutine ispndf ( npf, rspv, cv, ndim )
c ======================================================================
c
c
c        programmer     Ronald van Nooyen
c        version 1.1    date 13-12-94  (New layout)
c        version 1.0    date 24-06-94
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Initializes a set of arrays that describe the interpolation routines
c     needed to determine the values and partial derivatives of coefficients,
c     variables and coordinates in 4 (2d) or 8 (3D) points in a grid cell.
c ********************************************************************
c
c                      KEYWORDS
c
c    finite_difference_formulas
c    interpolation_formulas
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer npf(0:3,0:4,1:8), rspv(1:3,1:8,0:3,0:4,1:8), ndim
      double precision cv(1:8,0:3,0:4,1:8)

c     cv        i    reserved
c     ndim      i    Dimension of space (2 or 3)
c     npf       i    reserved
c     rspv      i    reserved
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS

      integer vartyp, pnttyp, m, action, alphad, powtwo, indexoff,
     +        ioffst(2,3)
      logical split(3)
      double precision factor(2,3)

c     action    Indicates the type of action to be carried out
c               Possible values:
c               0:   interpolation or extrapolation for point value
c               1 to ndim: interpolation or extrapolation for
c                    partial derivative in direction action
c     alphad    Direction index
c     factor    factors for linear interpolation or
c               difference formula in direction compon
c     indexoff  Index in array ioffst or array factor.
c               indexoff refers to the firts index to be used in these arrays
c     ioffst    Contains the offsets for the linear interpolation
c               or difference formula with respect to the global
c               numbering
c     m         General loop variable
c     pnttyp    reserved
c     powtwo    Help variable to compute power of two
c     split     If true, two points are needed for the formula,
c               otherwise only one is needed
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or :
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISPNDF1   Make tables for interpolation and finite difference formulas in
c               one direction only
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    METHOD
c
c        A formula is constructed by taking the cartesian product of
c        its component one dimnesional formulas.
c
c        e.g. a finite difference in direction alpha of the pressure
c        in a vertex is
c        constructed by combining a 1D interpolation formulas in the
c        directions orthogonal to alpha and a
c        1D finite difference formula in direction alpha.
c
c
c        Example: For alpha=1, we have in the above case, three
c        separate formulas.
c
c        offset -1 with weight -1 and offset 0 with weight 1 in
c        direction 1
c        offset -1 with weight 0.5 and offset 0 with weight 0.5 in
c        direction 2
c        offset -1 with weight 0.5 and offset 0 with weight 0.5 in
c        direction 3
c
c        The cartesian product give the following set of 8 points in
c        the formula
c
c        {-1,0} X {-1,0} X {-1,0} = {(-1,-1,-1),(0,-1,-1),...,(0,0,0)}
c
c        The coefficient in each point is the product of the
c        coefficients of the selected directional components in
c        the 2D formulas.
c
c                    PSEUDO CODE
c
c    Initialize the arrays to zero
c    For action := 0 (1) ndim do
c       For vartyp := 0 (1) ndim+1 do
c          For pnttyp := 1 (1) ndim^2 do
c             npf(action,vartyp,pnttyp) := 1
c             For direc := 1 (1) ndim do
c                split(direc) := interpolation in direction direc needs two
c                                points
c                fill factor(.,direc) with weights in direction direc
c                fill ioffst(.,direc) with shifts in direction direc
c                if ( split(direc)=true ) then
c                   npf(action,vartyp,pnttyp) := 2*npf(action,vartyp,pnttyp)
c             For m := 1 (1) npf(action,vartyp,pnttyp) do
c                powtwo := 0
c                cv(m,action,vartyp,pnttyp) := 1
c                For direc := 1 (1) ndim do
c                   if ( split(direc)=true ) then
c                      indexoff := 1 or 2 according to mod(m-1/2^powtwo,2)+1
c                      rspv(direc,m,action,vartyp,pnttyp) :=
c                         ioffst(indexoff,direc)
c                      cv(m,action,vartyp,pnttyp) := factor(indexoff,direc) *
c                         cv(m,action,vartyp,pnttyp)
c                      powtwo := powtwo+1
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- Initialize the arrays to zero

      do 140 pnttyp = 1, 8
         do 130 vartyp = 0, 4
            do 120 action = 0, 3
               npf(action,vartyp,pnttyp) = 0
               do 110 m = 1, 8
                  cv(m,action,vartyp,pnttyp) = 0d0
                  do 100 alphad = 1, 3
                     rspv(alphad,m,action,vartyp,pnttyp) = 0
100               continue
110            continue
120         continue
130      continue
140   continue

c     --- Determine maximum number of points that may occur in an
c         interpolation or finite difference formula

      do 300 action = 0, ndim

c     --- Loop to derive point value and directional derivative
c         formulas

         do 260 vartyp = 0, ndim+1

c        --- Loop over dependent variable type

            do 250 pnttyp = 1, 2**ndim

c           --- Loop over point type
c               Initialize the total number of points involved in
c               determining the result of action act on
c               variable vartyp in point pnttyp

               npf(action,vartyp,pnttyp) = 1

               do 200 alphad = 1, ndim

c              --- Determine the one-dimensional linear
c                  interpolation or extrapolation formula needed in
c                  coordinate direction alphad to perform the
c                  interpolation

                  call ispndf1 ( action, alphad, vartyp, pnttyp,
     +                           ioffst(1,alphad), factor(1,alphad),
     +                           split(alphad), ndim )

                  if ( split(alphad) )
     +               npf(action,vartyp,pnttyp) =
     +                  2*npf(action,vartyp,pnttyp)

200            continue

               do 220 m = 1, npf(action,vartyp,pnttyp)

c              --- Expand the one-dimensional formulas

                  powtwo = 0

                  cv(m,action,vartyp,pnttyp) = 1d0

                  do 210 alphad = 1, ndim

                     if ( split(alphad) ) then

c                    --- interpolation needs two points

                        indexoff = mod((m-1)/(2**powtwo),2)+1
                        rspv(alphad,m,action,vartyp,pnttyp)  =
     +                       ioffst(indexoff,alphad)

                        cv(m,action,vartyp,pnttyp)  =
     +                       cv(m,action,vartyp,pnttyp) *
     +                       factor(indexoff,alphad)

                        powtwo = powtwo+1

                     end if

210               continue

220            continue

250         continue

260      continue

300   continue

      end
