      subroutine isvecup1 ( alphad, ni, nj, nvirtual, veloc, rho,
     +                      vecup1, locdx, iconserv, ibinfor, ibipoint )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.0    date 18-01-1999 points outside region changed
c                                       Extra parameters
c        version 1.2    date 10-12-1998 iconserv added
c        version 1.1    date 09-12-1998 invjac removed
c        version 1.0    date 14-08-1998
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute help variable, in cell centre, for first order
c   conservative upwind.
c   Wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    upwind
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer alphad, ni, nj, nvirtual, locdx(2), iconserv, ibipoint(*),
     +        ibinfor(*)
      double precision veloc(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2),
     +                  rho(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                  vecup1(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iconserv  i    If 1 the conservative scheme is used, otherwise (0) the
c                    non-conservative one
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rho       i    Estimate of the asymptotic convergence rate
c     vecup1    o    Upwind help vector 1, rho*V/sqrt(g) in cell centre
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer btype, ipibinf, ip, jp

c     btype     Type of boundary condition
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c   In case of the Wesbeek discretization we have the following expressions
c   for the convective terms (alpha cell):
c
c     (alpha)           1  |i+1/2,j          2  |i,j+1/2
c    a  (i,j)  . ( rho V u |         +  rho V u |         )
c                          |i-1/2,j             |i,j-1/2
c
c                   1           2
c   with u = (a    V   +  a    V ) / sqrt(g)
c              (1)         (2)
c
c
c   In the Picard linearization we use V at the old level
c
c   So the contributions to the matrix become:
c
c                                                  (alpha)      _1
c   Picard: locdx = (+/-1, 0):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _1
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c                                                  (alpha)      _2
c           locdx = (0, +/-1):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _2
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c   For the conservative first order upwind method in this
c                             _
c   routine the factor : rho  V / sqrt(g) named VECUP1  is computed
c
c   in the cell center.
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- initialize vec1

      do ip = 1-nvirtual, ni+1+nvirtual
         do jp = 1-nvirtual, nj+1+nvirtual
            vecup1(ip,jp) = 0d0
         end do
      end do

      if ( locdx(1).ne.0 ) then

c     --- use V1

         do ip = 0, ni+2
            do jp = 0, nj+2
               vecup1(ip,jp) = rho(ip,jp)*veloc(ip,jp,1)
            end do
         end do

c        --- for points outside the region take factor from inside

         if ( alphad.eq.1 .and. iconserv.eq.0 ) then

c        --- Non conservative scheme
c            First side 1, points with i = 0, are outside the region
c            Only the points that do not have periodical or multiblock
c            boundary conditions should be treated as special

            ipibinf = ibipoint(1)+nvirtual-1
            do jp = 0, nj+2
               btype = ibinfor(ipibinf+2*jp-1)
               if ( btype.lt.90 ) vecup1(0,jp) = vecup1(1,jp)
            end do

c           --- Next side 2, points with i = ni+1, are outside the region if
c               locdx(1) > 0

            if ( locdx(1).gt.0 ) then
               ipibinf = ibipoint(2)+nvirtual-1
               do jp = 0, nj+2
                  btype = ibinfor(ipibinf+2*jp-1)
                  if ( btype.lt.90 ) vecup1(ni+1,jp) = vecup1(ni,jp)
               end do
            end if
         end if

      else if ( locdx(2).ne.0 ) then

c     --- use V2

         do ip = 0, ni+2
            do jp = 0, nj+2
               vecup1(ip,jp) = rho(ip,jp)*veloc(ip,jp,2)
           end do
         end do

c        --- for points outside the region take factor from inside

         if ( alphad.eq.2 .and. iconserv.eq.0 ) then

c        --- Non conservative scheme
c            First side 3, points with j = 0, are outside the region
c            Only the points that do not have periodical or multiblock
c            boundary conditions should be treated as special

            ipibinf = ibipoint(3)+nvirtual-1
            do ip = 0, ni+2
               btype = ibinfor(ipibinf+2*ip-1)
               if ( btype.lt.90 ) vecup1(ip,0) = vecup1(ip,1)
            end do

c           --- Next side 4, points with j = nj+1, are outside the region if
c               locdx(2) > 0

            if ( locdx(2).gt.0 ) then
               ipibinf = ibipoint(4)+nvirtual-1
               do ip = 0, ni+2
                  btype = ibinfor(ipibinf+2*ip-1)
                  if ( btype.lt.90 ) vecup1(ip,nj+1) = vecup1(ip,nj)
               end do
            end if
         end if

      end if

      end
