      subroutine isco2put ( iblock, iface, isubface, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf, nsubfaces,
     +                      subfaces, coor, vco )
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.0    date 30-11-1995 Guus: extracted from iscoget
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Put the virtual coordinates from an intermediate representation vco into
c     the virtual region of a certain subface of  a certain block.
c     These coordinates are numbered in the increasing i and j directions.
c     Coordinates are numbered in the tangential direction first, then
c     in a direction normal to the interface. The x coordinates occur
c     first then the y coordinates.
c     The intermediate representation may be treated as a multi-dimensional
c     array as follows:
c     vco( 1:nvirtual, 1:ndim, 1:number_of_coordinates_in_tangential_dir)
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, isubface, maxnsubf, nblocks,
     +        nfaces, ni, nj, nvirtual, nsubfaces(nblocks,nfaces),
     +        subfaces(nblocks,nfaces,maxnsubf,*)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 vco(nvirtual,2,*)

c     coor     i/o   reserved
c     iblock    i    Block sequence number
c     iface     i    Face number
c     isubface  i    Subface number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c     vco       i    Work array in which the co-ordinates to be copied are
c                    stored
c                    vco has dimension: vco(1:nvirtual, 1:ndim, 1:nijk+1)
c                    with nijk max(ni,nj,nk)
c                    Array vco(i,m,n) contains the differences between points
c                    at a distance from the boundary and the corresponding
c                    points on the boundary.
c                    i:  the distance normal to the subface
c                    m:  refers tot the x (m=1), y (m=2), or z (m=3)
c                        co-ordinate
c                    n:  the distance along the subface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i1, i2, ihigh, ii, ilow, inorm, istep,
     +        j1, j2, jj, jstep, k, tsignmb, ishift, jshift

c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     inorm     Loop variable
c     ishift    Shift in i-direction
c     istep     Step in the i-direction
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     jj        Loop variable
c     jshift    Shift in j-direction
c     jstep     Step in the j-direction
c     k         General loop variable
c     tsignmb   If the tangential direction of the subface coincides with
c               that of its neighbouring block tsignmb = 0, otherwise
c               tsignmb = 1
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1600   iblock out of range   (iblock,nblocks)
c    1601   iface out of range    (iface,nfaces)
c    1602   isubface out of range (isubface,nsubfaces)
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     perform some checks
c     Compute the i and j bounds for the co-ordinates at the boundary
c     Compute the co-ordinates at boundary from vco
c
c     Explanation,  suppose we want to couple the block sketched in
c     isco2get at the right-hand side of the following block:
c
c
c     -------------------------------------------------------------------------
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |     |     |
c     -------------------------------------------------------------------------
c     1     2     3                                        n-1    n    n+1   n+2
c
c
c     So we need two virtual rows of co-ordinates
c     In vco we have stored delta x, respectively 2 delta x
c
c     The virtual co-ordinates in n+1 are equal to the co-ordinates in
c     n plus delta x en in n+2 equal to n plus 2 delta x
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- first perform some checks

      if ( iblock.lt.1 .or. iblock.gt.nblocks ) then

         call iseropen('isco2put')
         call iserrint(iblock, 1)
         call iserrint(nblocks, 2)
         call iserrsub ( 1600, 2, 0, 0)
         call iserclos('isco2put')

      end if

      if ( iface.lt.1 . or. iface.gt.nfaces ) then

         call iseropen('isco2put')
         call iserrint(iface, 1)
         call iserrint(nfaces, 2)
         call iserrsub ( 1601, 2, 0, 0)
         call iserclos('isco2put')

      end if

      if ( isubface.lt.1 .or.
     +     isubface.gt.nsubfaces(iblock,iface) ) then

         call iseropen('isco2put')
         call iserrint(isubface, 1)
         call iserrint(iblock,   2)
         call iserrint(iface,    3)
         call iserrint(nsubfaces(iblock, iface), 4)
         call iserrsub ( 1602, 4, 0, 0)
         call iserclos('isco2put')

      end if

c     --- Compute the i and j bounds for the co-ordinates at the boundary

      ilow = subfaces(iblock, iface, isubface, 1)
      ihigh = subfaces(iblock, iface, isubface, 2) + 1
      tsignmb = mod(subfaces(iblock,iface,isubface,8), 2)

      istep = 1
      jstep = 1

      if ( iface.eq.ileft ) then

         ii = 1
         if ( tsignmb.eq.1 ) then
            jstep = -1
            j1    = ihigh
            j2    = ilow
         else
            j1 = ilow
            j2 = ihigh
            jstep = 1
         end if

      else if ( iface.eq.iright ) then

         ii = ni + 1
         if ( tsignmb.eq.1 ) then
            jstep = -1
            j1    = ihigh
            j2    = ilow
         else
            jstep = 1
            j1 = ilow
            j2 = ihigh
         end if

      else if ( iface.eq.idown ) then

         jj = 1
         if ( tsignmb.eq.1 ) then
            istep = -1
            i1    = ihigh
            i2    = ilow
         else
            istep = 1
            i1 = ilow
            i2 = ihigh
         end if

      else

         jj = nj + 1
         if ( tsignmb.eq.1 ) then
            istep = -1
            i1    = ihigh
            i2    = ilow
         else
            istep = 1
            i1 = ilow
            i2 = ihigh
         end if

      end if

c     --- Compute coordinates by adding the differences stored in vco

      do inorm = 1, nvirtual

         if ( iface.eq.ileft ) then

            ishift = -inorm

         else if ( iface.eq.iright ) then

            ishift = inorm

         else if ( iface.eq.idown ) then

            jshift = -inorm

         else

            jshift = inorm

         end if

         if ( iface.eq.iright .or. iface.eq.ileft ) then

c        --- right or left face, loop over j

            do k = 1, 2
               do jj = j1, j2, jstep

c              --- write(6,*)vco(inorm,k,jstep*(jj-j1)+1)
c                  write(6,*) coor(ii,jj,k)

                  coor(ii+ishift,jj,k) = vco(inorm,k,jstep*(jj-j1)+1) +
     +                                   coor(ii,jj,k)
               end do
            end do

         else

c        --- bottom or top face, loop over i

            do k = 1, 2
               do ii = i1, i2, istep
                  coor(ii,jj+jshift,k) = vco(inorm,k,istep*(ii-i1)+1) +
     +                                   coor(ii,jj,k)
               end do
            end do

         end if

      end do

      end
