      subroutine isco3get ( iblock, iface, isubface, ni, nj, nk,
     +                      nvirtual, nblocks, nfaces, maxnsubf,
     +                      subfaces, coor, vco, maxnijk, corner1 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 24-06-1997 extension with corners
c        version 1.0    date 05-12-1995 3d extension of isco2get
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Get the coordinates from a certain subface of a certain block that are
c     transported to neighbouring blocks into an intermediate representation.
c     These coordinates are numbered in the increasing i and j directions.
c     Coordinates are numbered in the tangential direction first, then
c     in a direction normal to the interface. The x coordinates occur
c     first then the y coordinates.
c     The intermediate representation may be treated as a multi-dimensional
c     array as follows:
c     vco( 1:nvirtual, 1:ndim, 1:number_of_coordinates_in_tangential_dir)
c **********************************************************************
c
c                       KEYWORDS
c
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, isubface, maxnsubf, nblocks,
     +        nfaces, ni, nj, nk, nvirtual, maxnijk
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 vco(nvirtual,3,maxnijk+1,maxnijk+1),
     +                 corner1(1:3)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     corner1   o    First corner of neighbouring block
c     iblock    i    Block sequence number
c     iface     i    Face number
c     isubface  i    Subface number
c     maxnijk   i    Max(ni,nj,nk) over all blocks
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c     vco       o    Work array in which the co-ordinates to be copied are
c                    stored
c                    vco has dimension: vco(1:nvirtual, 1:ndim, 1:nijk+1)
c                    with nijk max(ni,nj,nk)
c                    Array vco(i,m,n) contains the differences between points
c                    at a distance from the boundary and the corresponding
c                    points on the boundary.
c                    i:  the distance normal to the subface
c                    m:  refers tot the x (m=1), y (m=2), or z (m=3)
c                        co-ordinate
c                    n:  the distance along the subface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i1, i2, ihigh, ii, ilow, inorm, j1, j2, jj, k1, k2, kk,
     +        ishift, idim, jlow, jhigh

c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     idim      Loop variable used for dimension
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     inorm     Loop variable
c     ishift    Shift in i-direction
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     jhigh     Upper j-index for a subface
c     jj        Loop variable
c     jlow      Lower j-index for a subface
c     k1        Lower index number along third direction of a face or edge
c     k2        Upper index number along third direction of a face or edge
c     kk        Loop variable
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     none
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Compute the i and j bounds for the co-ordinates at the boundary
c     Put the co-ordinate differences into vco
c
c     The reason that vco contains the differences rather than the actual
c     values is only because periodical boundary conditions may be
c     treated by coupling of two sides of the boundary onto each other
c     As a consequence it is necessary that copying must be performed
c     such that cells at both sides of the region are again periodical
c     This can be done by computing the distances rather than by copying
c
c     See subroutine isco2get for an explanation
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute the i, j and k bounds for the co-ordinates at the boundary

      ilow = subfaces(iblock, iface, isubface, 1)
      ihigh = subfaces(iblock, iface, isubface, 2) + 1
      jlow = subfaces(iblock, iface, isubface, 3)
      jhigh = subfaces(iblock, iface, isubface, 4) + 1
      if ( iface.eq.1 ) then

         ii = 1
         j1 = ilow
         j2 = ihigh
         k1 = jlow
         k2 = jhigh

      else if ( iface.eq.2 ) then

         ii = ni+1
         j1 = ilow
         j2 = ihigh
         k1 = jlow
         k2 = jhigh

      else if ( iface.eq.3 ) then

         jj = 1
         i1 = jlow
         i2 = jhigh
         k1 = ilow
         k2 = ihigh

      else if ( iface.eq.4 ) then

         jj = nj+1
         i1 = jlow
         i2 = jhigh
         k1 = ilow
         k2 = ihigh

      else if ( iface.eq.5 ) then

         kk = 1
         i1 = ilow
         i2 = ihigh
         j1 = jlow
         j2 = jhigh

      else

         kk = nk+1
         i1 = ilow
         i2 = ihigh
         j1 = jlow
         j2 = jhigh

      end if

c     --- fill vco with difference of coordinates to be transported
c         and boundary values of coor (needed for periodic boundary
c         conditions)

      do inorm = 1, nvirtual

         if ( mod(iface,2).eq.1 ) then

            ishift = inorm

         else

            ishift = -inorm

         end if

         if ( iface.le.2 ) then

c        --- faces 1 and 2, loop over j and k

            do idim = 1, 3
               corner1(idim)=coor(ii,j1,k1,idim)
            end do

            do idim = 1, 3
               do jj = j1, j2
                  do kk = k1, k2
                     vco(inorm,idim,jj-j1+1,kk-k1+1) =
     +                   coor(ii+ishift,jj,kk,idim) -
     +                   coor(ii,jj,kk,idim)
                  end do
               end do
            end do

         else if ( iface.le.4 ) then

c        --- faces 3 and 4, loop over i and k

            do idim = 1, 3
               corner1(idim)=coor(i1,jj,k1,idim)
            end do

            do idim = 1, 3
               do ii = i1, i2
                  do kk = k1, k2
                     vco(inorm,idim,ii-i1+1,kk-k1+1) =
     +                   coor(ii,jj+ishift,kk,idim) -
     +                   coor(ii,jj,kk,idim)
                  end do
               end do
            end do

         else

c        --- faces 5 and 6, loop over i and j

            do idim = 1, 3
               corner1(idim)=coor(i1,j1,kk,idim)
            end do

            do idim = 1, 3
               do ii = i1, i2
                  do jj = j1, j2
                     vco(inorm,idim,ii-i1+1,jj-j1+1) =
     +                   coor(ii,jj,kk+ishift,idim) -
     +                   coor(ii,jj,kk,idim)
                  end do
               end do
            end do

         end if

      end do

      end
