      subroutine ischsrt0 ( keysrt, nlo, nstr, nkey, kgrad, khelp,
     +                      nscrch )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        chsrt0 version 2.3)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    ischsrt0 merges two partially sorted array parts. The sortkey is in the
c    array keysrt, the permutation that partially sorts the array is in kgrad.
c    Merge operates on kgrad only, leaving keysrt unchanged. Effectively,
c    kgrad will be merged, using keysrt to decide what comes first.
c    The parts to be merged, are adjacent in kgrad, the first part starts
c    at nlo, the second part starts at nlo + nstr / 2 unless this is larger
c    than nkey, in which case the second part is empty and the merge is
c    finished. On output kgrad (nlo) to kgrad (min(nlo + nstr - 1, nkey)
c    contains the permutation that totally sorts this array part.
c    This version uses the work array khelp as a scratch pad.
c    REMARK: The efficiency of chsrt0 may be enhanced by enlarging
c            the dimension of the array khelp to  nkey
c            Taking a smaller value for NSCRCH reduces the efficiency.
c ********************************************************************
c
c                      KEYWORDS
c
c   copy
c   merge
c   sort
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nlo, nstr, nkey, NSCRCH, kgrad(*), khelp(*), keysrt(*)

c     keysrt   i   Sortkey (integer) on array to be sorted.
c     kgrad   i/o  Contains permutation index that sorts keysrt
c                  in ascending order on output.
c     khelp        Buffer used to temporarily store an already
c                  merged array part. From khelp it will be copied to kgrad.
c     nkey     i   Number of entries in array to be sorted
c     nlo      i   (Indirect) starting address of array parts to be merged
c     nscrch   i   Declared dimension of khelp
c     nstr     i   Total number of elements of the two array parts to
c                  be merged.
c ********************************************************************
c
c                      COMMON BLOCKS
c
c    -
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ilo1, ilo2, ihi1, ihi2, imoves, imovsh, nmoves, icnt,
     +        icnth
      logical ready

c     icnt      Counter to positions in buffer
c     icnth     Auxiliary variable carrying counter to icnt outside
c               the do loop
c     ihi1      Upper bound of first array part
c     ihi2      Upper bound of second array part
c     ilo1      Address in kgrad of first element of first array part not
c               yet merged
c     ilo2      Address in kgrad of first element of second array part not
c               yet merged
c     imoves    Counter to nmoves
c     imovsh    Temporary storage of imoves
c     nmoves    Number of times khelp has to be emptied in kgrad.
c     ready     Indicates if a while loop is finished or not
c ********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISINCOPY  to copy vectors. LAST entry FIRST.
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c    Algorithm:
c
c    define krgad (ilo1) ::= ind1, kgrad (ilo2) ::= ind2.
c    keysrt (ind1) is compared with keysrt (ind2).
c    The index of the smaller one is placed in the buffer, and the buffer
c    count  increased. The corresponding ilo is increased and the process
c    is repeated until either
c
c    1. The buffer is full.
c    2. one of the two array parts is exhausted.
c
c    If (1) occurs, the remainder of the lowest of the two array parts in
c    kgrad is moved exactly over the number of places that has become free
c    in the highest of the two array parts. This exactly frees the number
c    of places necessary to copy the buffer into kgrad, after which the
c    buffer count is reset to 1 and the process continues.
c
c    If (2) occurs, the process is basically finished, except that
c    the mess has to be cleaned up. If the highest part is exhausted
c    the lowest part has to be moved to free sufficient places to copy
c    the contents of the (partially filled) buffer into kgrad.
c    If the lowest part is exhausted this is not necessary, since the
c    space to receive the buffer is already there.
c    Finally the remainder of the buffer is copied into kgrad.
c
c    Remark:
c
c    Array keysrt:
c
c      Sorts keysrt in ascending order on output.
c      sorts keysrt. That is on input kgrad (ilo) to
c      kgrad (ilo + nstr / 2 - 1) and kgrad (ilo + nstr / 2)
c      to kgrad (ilo + nstr - 1) contain indices to partially
c      sort sections of keysrt. On output kgrad (ilo) to
c      kgrad (ilo + nstr - 1) contains indices to sort
c      the whole section. If ilo + nstr / 2 or ilo + nstr - 1
c      should be > nkey, nkey is taken as upper bound.
c ======================================================================
c
      ilo1 = nlo
      ilo2 = nlo + nstr/2
      ihi1 = ilo2 - 1
      ihi2 = ilo1 + nstr - 1
      ihi2 = min (ihi2, nkey)
      ready = .false.
      if ( ihi2.ge.ilo2 ) then

c     --- Calculate number of times the total array part to be sorted
c         fits into the auxiliary buffer. (remainder included). This is
c         the number of moves to be made.

         nmoves = (nstr - 1) / NSCRCH + 1
         do 120 imoves = 1, nmoves
            do 100 icnt = 1, NSCRCH

c           --- The first and second array part are compared entry by entry.
c               The index of the lowest of the two is stored in the buffer.

               if (keysrt (kgrad (ilo1)).gt.keysrt (kgrad (ilo2))) then
                  khelp (icnt) = kgrad (ilo2)
                  ilo2 = ilo2 + 1

c                 --- test if this array part exhausted

                  if ( ilo2.gt.ihi2 ) then

c                 --- Yes. Merge ended. Save icnt and imoves out of the do loop.

                     icnth = icnt
                     imovsh = imoves

c                    --- If the higher part exhausts, the remainder of the lower
c                        part has to be shifted first, so as to occupy the
c                        highest positions.
c                        After that the contents of the buffer can be copied
c                        onto the part just made free.

                     call isincopy ( kgrad, kgrad, ilo1, ihi2+ilo1-ihi1,
     +                               ihi1-ilo1+1, -2 )
                     call isincopy ( khelp, kgrad, 1,
     +                               nlo+NSCRCH*(imovsh-1), icnth, -2 )
                     ready = .true.

                  end if

               else

                  khelp (icnt) = kgrad (ilo1)
                  ilo1 = ilo1 + 1

c                 --- test if this array part exhausted

                  if ( ilo1.gt.ihi1 ) then

c                 --- Yes. Merge ended. Save icnt and imoves out of the do loop.

                     icnth = icnt
                     imovsh = imoves

c                    --- During merging either array part may get exhausted.
c                        If the lowest part exhausts, the contents of the buffer
c                        have to be copied onto the space previously occupied
c                        by the exhausted part.
c                        (There will be just enough room to do so.)

                     call isincopy ( khelp, kgrad, 1,
     +                               nlo+NSCRCH*(imovsh-1), icnth, -2 )
                     ready = .true.

                  end if

               end if
               if ( ready ) go to 1000

100         continue

c           --- The buffer is full. Contents of kgrad (lowest part) have to be
c               moved up, to make room for the contents of the buffer. Then the
c               buffer is copied onto kgrad.

            call isincopy ( kgrad, kgrad, ilo1, ilo1+ilo2-ihi1-1,
     +                      ihi1-ilo1+1, -2 )
            call isincopy ( khelp, kgrad, 1, nlo+(imoves-1)*NSCRCH,
     +                      NSCRCH, -2 )

c           --- ilo1 & ihi1 get the values of the shifted part.

            ilo1 = ilo1 + ilo2 - ihi1 - 1
            ihi1 = ilo2 - 1

120      continue

      end if

1000  continue
      end
