      subroutine gmresr(n,amat,rhsd,usol,solutv,eps1,eps2,itmax,work,
     +                  icontr,infmat,prec,nprec,ndim,nconct,iinmtvc)
c ======================================================================
c
c     programmer       Kees Vuik
c     version 1.8      date 21-07-1999 Guus: set bmin
c     version 1.7      date 21-11-1997 Kees K. iinmtvc added
c     version 1.6      date 31-10-1997 Kees V. changes for uncoupled
c     version 1.5      date 04-10-1995 Kees K. solutv added
c     version 1.4      date 04-04-1995 Guus replace mtvc1 by ismatvec
c     version 1.3      date 06-04-1993 C. Vuik Initilization of some
c                                      constants and giving a warning
c                                      if eps is too small. Finally a
c                                      start with |r0| = 0 is possible.
c     version 1.2      date 06-08-1992
c     developed at     Convex,HP700
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The subroutine GMRESR is used to solve an unsymmetric system of
c     linear equations of the shape A.x = f.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioning
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c     amat    i   Matrix from the equations to be solved.
c     eps1    i   Determine the accuracy of the final approximation. The
c     eps2    i   termination criterion is based upon
c                           eps1 + eps2 * ( A x  - f ) .
c                                              0
c     icontr i-o  Integer array in which information about the solution
c                 process must be given by the user. On output information
c                 about the solution process is provided. ICONTR must be
c                 filled as follows:
c             1 i Parameter controlling the preconditioning. Possible
c                 values are:
c                  0: No preconditioning.
c                  1: Diagonal preconditioner.
c                 -1: Diagonal postconditioner.
c                  2: ILUD preconditioner, with Eisenstat implemen-
c                     tation.
c                 -2: ILUD postconditioner.
c                  3: ILU preconditioner.
c                 -3: ILU postconditioner.
c             2 i Control parameter indicating the amount of output
c                 required. Possible values:
c                 <0: No output.
c                  1: Only fatal errors will be printed.
c                  2: Additional information about the iteration is printed.
c                  3: Gives a maximal amount of output concerning the
c                     iteration process.
c             3 o At output an error indication is stored. Possible values:
c                 0: No fatal errors.
c                 3: The number of iterations exceeds itmax.
c             4 o Contains at output the actual number of iterations
c                 performed.
c             5 i Parameter indicating the file number to which output
c                 should be written.
c             6 i Parameter controlling the size of the Krylov-subspaces,
c                 thus determining the restarts of the iteration process.
c             7 i In the GMRESR process the residual is made orthogonal
c                 to icontr(7) vectors.
c             8 i The degree of the preconditioning polynomial is less
c                 than icontr(8). Icontr(8) should be less than 100.
c             9 i The method for the innerloop.
c                 Possible values:
c                 0: LSQR
c                 1: CGS
c                 2: GMRES
c                 3: GCR
c                 4: CGSTAB
c     iinmtvc i   reserved
c     infmat  i   Integer array with information of the matrix structure,`
c                 to be used in matrix-vector multiplication subroutine
c     itmax   i   The maximum number of iterations to be performed.
c     n       i   The number of rows in the matrix A.
c     nconct  i   Maximal number of connections in one row of the
c                 matrix.
c     ndim    i   Integer indicating the amount of unknowns in every
c                 gridpoint. In the momentum equations ndim = 2 or 3
c                 whereas in the pressure and transport equations
c                 ndim = 1.
c     nprec   i   Number of diagonals which are used in the
c                 preconditioning.
c     prec    i   Array which contains the preconditioning matrix.
c     rhsd    i   Vector containing the right-hand side vector of the
c                 system of equations.
c     solutv  i   Work array for solut with virtual points
c     usol   i-o  Solution vector of length n. On input the array should
c                 contain a starting vector. At output the array contains
c                 the last iterate, which is an approximation to the
c                 solution of the system.
c     work    o   Work array of size n * (2*icontr(7)+qmr+10), where qmr
c                 is given in the follwing table:
c                 icontr(9)   qmr
c                    0         5
c                    1         7
c                    2         icontr(8)
c                    3         ?
c                    4         ?
c                 On output the first n elements contain the residual vector.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c     bmin     Variable used in the J&R truncation strategy.
c     bnorm    Contains the norm of the righthandside vector after
c              preconditioning.
c     eps      Required accuracy (including relative and absolute error).
c     epsmac   The machine precision.
c     i        Loop variable for the inner iteration.
c     ihulp    Auxiliary variable.
c     imax     Maximal value for i.
c     imin     Variable used in the J&R truncation strategy.
c     iprint   Parameter indicating the file number to which output should
c              be written.
c     iter     The current iteration number.
c     itnmin   Auxiliary variable.
c     j        Counting variables.
c     mpost    Maximal number of iteration steps in the call of htim.
c     ntrunk   Number of vectors used in the truncation process.
c     rnorm    Norm of the residual vector.
c     start    Logical value indicating whether a restart is made.
c     t        Auxiliary variable.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     daxpy    Computes a vector + a constant * other vector. This
c              is a BLAS routine.
c     dcopy    Copy a vector to another vector. This is a BLAS
c              routine.
c     ddot     Computes an inner product. This is a BLAS routine.
c     dinvl    Computes the multiplication of the inverse of a lower
c              triangular matrix, stored in arrays prec and amat,
c              and a vector.
c     dinvu    Computes the multiplication of the inverse of a upper
c              triangular matrix, stored in arrays prec and amat,
c              and a vector.
c     dnrm2    Calculates the 2-norm of a given vector. This is
c              a BLAS routine.
c     htim     Calculates a suitable search direction for the GMRESR
c              method. Originally it consists of a call of GMRES. At
c              this moment it can also use other iterative methods.
c              The choice depends on the contents icontr(9).
c     ismatvec Computes the product of the sparse matrix, stored in AMAT,
c              and a vector.
c     prevc    Computes the product of the preconditioner, stored in
c              amat and prec.
c     prires   Provides information about the solution process; the amount
c              of output is determined by the value of ICONTR(5).
c **********************************************************************
c
c
c **********************************************************************
c
c                       I/O
c
c     There is an I/O statement with label 1000.
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine GMRESR is used to solve an unsymmetric system of
c     linear equations of the shape A.x = f. Starting from an initial
c     vector, in each iteration a vector is sought which minimizes the
c     residual vector A.x-f in a certain Krylov-subspace. The process
c     restarts after a fixed number of iteration, with the latest approx-
c     imation as a new starting vector. Furthermore, only ntrunk vectors
c     are used to orthogonalize the search direction.
c
c     The method is described in:
c
c     GMRESR: a family of nested GMRES methods,
c     H.A. van der Vorst & C. Vuik,
c     TUD Report 91-80.
c
c     The work array can be interpreted as follows:
c
c       work(.,j)          contains u(j),
c       work(.,ntrunk+j)   contains c(j),
c       work(.,2*ntrunk+3) contains the residual,
c
c     where the vectors u(j), and c(j) are given on page 3 of TUD
c     Report 91-80.
c
c **********************************************************************
c
c
c **change1**  27-07-1992 C. Vuik
c
c reason    :  To implement the Eisenstat trick.
c
c references:  H.A. van der Vorst,
c              Journal of Computational Physics 44 pp.1-19 (1981).
c
c              S.C. Eisenstat,
c              SIAM J. Sci. Stat. Comput. 2 pp.1-4 (1981).
c
c **change1** end
c
c **change2**  27-07-1992 C. Vuik
c
c reason    :  To avoid underflow in the solution of a lower triangular
c              matrix.
c
c solution  :  We take as start solution usol = abs(rhsd)*10^-34.
c
c **change2** end
c
c
c     Input:
c
c     The elements ICONTR(1:2), ICONTR(5:6), EPS1, EPS2, ITMAX must have got
c     a value, and the arrays AMAT, RHSD and USOL must have been filled by
c     the calling program.
c
c     Output:
c
c     The elements ICONTR(3:4) contain information about the iteration
c     process, the calculated solution vector is stored in USOL, and the
c     corresponding residual vector in WORK(.,1).
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Parameters:

      integer n,itmax,icontr(9), infmat(*), nprec,ndim,nconct,
     +        iinmtvc(*)
      double precision amat(n,*),rhsd(n),usol(n),eps1,eps2,work(n,*),
     +                 prec(n,*),solutv(*)

c     --- Local parameters:

      logical          start
      integer          i,imax,iter,j,ntrunk,itnmin,ihulp,
     +                 mpost,imin,iprint
      double precision rnorm,t,epsmac,eps,ddot,dnrm2,bmin,bnorm
      data epsmac/1.0d-16/

c     --- Determination of the maximum number of inner iterations.

      start = .true.
      iprint = icontr(5)
      icontr(4) = 0
      imax   = icontr(6)
      ntrunk = icontr(7)
      mpost  = icontr(8)
      bmin   = 0d0
      write(iprint,1000) imax,ntrunk,mpost
1000  format(' The GMRESR method restarts after ',i5,' iterations.',/,
     +       ' There are ',i5,' truncation vectors used.',/,
     +       ' The degree of the polynomial is less than',i5)

c     --- If icontr(1) equals 2 then the linear system is scaled by a
c         diagonal matrix, such that diag(L) = diag(U) = D = I.

      if(icontr(1).eq.2) then
         do j = 1, n
            rhsd(j) = rhsd(j)*prec(j,1)
         end do
      end if

c     --- change2**

      if(abs(icontr(1)).ge.2) then
         rnorm = dnrm2(n,rhsd,1)*1d-34
         if(rnorm.gt.0) then
            do j = 1, n
               usol(j) = usol(j)+rnorm
            end do
         end if
      end if

c     --- change2** end

      rnorm = 0
      eps   = 0
      if(icontr(1).eq.2) then

c     --- change1**

         call dinvl(rhsd,work(1,2),amat,n/ndim,ndim,
     +              nconct,prec,nprec,infmat)

c        --- change1** end

      else
         if(icontr(1).gt.0) then
            call prevc(n,rhsd,work(1,2),amat,ndim,nconct,
     +                 prec,nprec,infmat,icontr(1))
         else
            call dcopy(n,rhsd,1,work(1,2),1)
         end if
      end if
      bnorm = dnrm2(n,work(1,2),1)

c     --- Main iteration loop.

      do iter = 1, itmax

c     --- Compute residual vector and its norm in case of restart.

         if (start) then
           i=0



           call ismatvec(amat,usol,solutv,work(1,2*ntrunk+3),infmat,
     +                   icontr, iinmtvc)

           do j = 1, n
              work(j,2*ntrunk+4) = rhsd(j)-work(j,2*ntrunk+3)
           end do

c          --- change1**

           if(icontr(1).eq.2) then
              call dinvl(work(1,2*ntrunk+4),work(1,2*ntrunk+3),
     +                    amat,n/ndim,ndim,nconct,prec,nprec,infmat)

c             --- change1** end

           else
              if(icontr(1).gt.0) then
                 call prevc(n,work(1,2*ntrunk+4),work(1,2*ntrunk+3),
     +                    amat,ndim,nconct,prec,nprec,infmat,icontr(1))
              else
                 call dcopy(n,work(1,2*ntrunk+4),1,work(1,2*ntrunk+3),1)
              end if
           end if
           rnorm = dnrm2(n,work(1,2*ntrunk+3),1)

c          --- Compute required accuracy.

           if (iter.eq.1) then
              eps = eps1 + eps2 * rnorm
              if(eps.lt.1000*epsmac*bnorm) then
                 write(iprint,1030) 1000*epsmac*bnorm, eps
1030             format(/,' *** WARNING ***             ',//,
     +                    ' The required eps is to small',/,
     +                    ' the machineprecision is     ',e15.5,/,
     +                    ' whereas eps is              ',e15.5)
                 eps = 1000*epsmac*bnorm
              end if
           end if

c          --- Check for convergence.

           if (rnorm.le.eps) then
              icontr(3)=0
              goto 500
           end if
           do j = 1, n
              work(j,2*ntrunk+4) = 0
           end do
           start = .false.
         end if

c        --- Print intermediate results and set iteration level.

         call prires(' GMRESR',rnorm,icontr,.false.)
         icontr(4) = iter

c        --- Inner iteration step, apply modified Gram-Schmid to form a
c            a new search direction.

         i=i+1
         itnmin = min(i-1,ntrunk)
         call htim(n,amat,
     +              work(1,2*ntrunk+3),work(1,2*ntrunk+1),
     +              solutv,
     +              work(1,2*ntrunk+2),eps,mpost,
     +              work(1,2*ntrunk+5),icontr,infmat,
     +              prec,nprec,ndim,nconct,iinmtvc)
         do j = 1, itnmin
            t = -ddot(n,work(1,ntrunk+j),1,
     +                work(1,2*ntrunk+2),1)
            call daxpy(n,t,work(1,j),1,work(1,2*ntrunk+1),1)
            call daxpy(n,t,work(1,ntrunk+j),1,work(1,2*ntrunk+2),1)
            if(abs(t).le.bmin.or.j.eq.1) then
               bmin = abs(t)
               imin = j
            end if
         end do
         t = 1.0/dnrm2(n,work(1,2*ntrunk+2),1)

c        --- trunclast strategy
c               ihulp = mod(i,ntrunk)
c               if(ihulp.eq.0) ihulp = ntrunk
c            truncfirst strategy
c               ihulp = min(i,ntrunk)
c            This is the Jackson & Robinson truncation strategy.
c            Reference
c            C.P. Jackson & P.C. Robinson,
c            A numerical study of various algorithms related to the precon-
c            ditioned conjugate gradient method,
c            Int. J. Num. Meth. Engng 21 pp.1315-1338 (1985)

         if(i.gt.ntrunk) then
            ihulp = imin
         else
            ihulp = i
         end if
         do j = 1, n
            work(j,ihulp) = work(j,2*ntrunk+1)*t
            work(j,ntrunk+ihulp) = work(j,2*ntrunk+2)*t
         end do

c        --- update x and r

         t = ddot(n,work(1,ntrunk+ihulp),1,
     +            work(1,2*ntrunk+3),1)
         call daxpy(n,t,work(1,ihulp),1,work(1,2*ntrunk+4),1)
         call daxpy(n,-t,work(1,ntrunk+ihulp),1,
     +               work(1,2*ntrunk+3),1)
         rnorm = dnrm2(n,work(1,2*ntrunk+3),1)

c        --- Check for termination of inner iterations.

         if (i.eq.imax .or. rnorm.lt.eps .or. iter .eq. itmax) then

c        --- Set restart.
c            change1**

            if(icontr(1).eq.2) then
               call dinvu(work(1,2*ntrunk+4),work(1,2*ntrunk+3),
     +                    amat,n/ndim,ndim,nconct,prec,nprec,infmat)

c              --- change1** end

            else
               if(icontr(1).ge.0) then
                call dcopy(n,work(1,2*ntrunk+4),1,work(1,2*ntrunk+3),1)
               else
                  call prevc(n,work(1,2*ntrunk+4),work(1,2*ntrunk+3),
     +                    amat,ndim,nconct,prec,nprec,infmat,icontr(1))
               end if
            end if
            call daxpy(n,1d0,work(1,2*ntrunk+3),1,usol,1)
            start = .true.
         end if
      end do

c     --- The iteration process terminates because the maximum number of
c         iterations is reached. Set error indicator.

      icontr(3) = 3

c     --- Normal termination of the iteration process.

500   call prires('GMRESR',rnorm,icontr,.true.)
      call dcopy(n,work(1,2*ntrunk+3),1,work,1)
      return
      end
