      subroutine isgaus( matrix, dpiter, rhs, jac, densup,
     +                  infmat,iinput, rinput, solut, mitergs,
     +                  tolbsec, nusol, iinmtvc, nsubfaces, subfaces)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.7    date 13-06-1999 Duncan: extension for low Mach
c     version      1.6    date 11-05-1999 Duncan: inclusion of periodical
c                                                 boundary conditions
c     version      1.5    date 29-03-1999 Duncan: inclusion of boundary points
c     version      1.4    date 17-11-1998 Duncan: total revision
c     version      1.3    date 22-07-1998 Duncan: extension for j
c                                                   direction
c                                         extension for density upwind
c     version      1.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.1    date 13-04-1998 Duncan: fixed bug for mscal=0
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compressible ISNaS subroutine isgaus does one sweep of non-linear
c     Gauss-Seidel to solve the non linear pressure correction equation
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer infmat(*), iinput(*), mitergs, nusol, iinmtvc(*),
     + nsubfaces(*), subfaces(*)
      double precision dpiter(nusol), matrix(*),
     +                  jac(*), densup(nusol,3,*),
     +                 rinput(*), solut(*), rhs(*), tolbsec

c     densup    i    coefficients of density upwind in pressure matrix
c     dpiter   i/o   work array for non linear Gauss-Seidel algorithm
c     iinmtvc   i    reserved
c     iinput    i    reserved
c     infmat    i    Integer information for the matrix
c     jac       i    sqrt(g) in pnttyp
c     matrix    i    reserved
c     mitergs   i    Maximum number of iterations in non-linear Gauss-Seidel
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     rhs       i    Right hand side  (any level)
c     rinput    i    reserved
c     solut     i    reserved
c     subfaces  i    reserved
c     tolbsec   i    Tolerance in the bisection step
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer j, nconct, iprincom, iprintim, ishift,
     +   ipinfmat, k, mscal, mcomupw, ni, nj, istartup, iconct,
     +   iprincav, totshift(9), periodic
      double precision cons1nl, cons2nl, cons3nl, ddp, matdp,
     +   dprside, dplside, l, dt, drhol, drhor, drhob, drhot, checklin,
     +   pres1tr, pres2tr, ppold, dpold
      logical boundaryp

c     boundaryp Point is on the boundary
c     checklin  check on linearty
c     cons1nl   Constant 1 in non linear equation
c     cons2nl   Constant 2 in non linear equation
c     cons3nl   Constant 3 in non linear equation
c     ddp       Half of the width of the search interval
c     dplside   Left side of the search interval for solution
c     dpold     pressure correction at previous time step
c     dprside   Right side of the search interval for solution
c     drhob     Increment in density at position 3 in pressure stencil
c     drhol     Increment in density at position 5 in pressure stencil
c     drhor     Increment in density at position 6 in pressure stencil
c     drhot     Increment in density at position 8 in pressure stencil
c     dt        Time step
c     iconct    Loop variable for loop over all connections in the matrix
c     ipinfmat  Starting address in array infmat
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ishift    Shift in i-direction
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     k         General loop variable
c     l         General loop variable
c     matdp     Contribution of DGdp to the rhs of the nonlinear equation
c     mcomupw   If zero the rho equation for compressible flow is solved by
c               a central scheme, otherwise by a upwind scheme
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     nconct    Number of connections in a row of the matrix
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     periodic  Periodic boundary conditions
c     ppold     pressure at previous time level
c     pres1tr   Lower transit pressure in equation of state
c     pres2tr   Upper transit pressure in equation of state
c     totshift  total shift in the matrix
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isbrzo, isstdls, isstdfl, isdstfll,
     +    isdstlss

c     ISBRZO     Solver for non linear scalar equations
c     ISDSTFLL   Derivative of dimensional equation of state
c     ISDSTLSS   Derivative of dimensionless equation of state
c     ISGSPER    Correct matrix shifts for periodic boundary conditions
c     ISSTDFL    dimensionfull equation of state
c     ISSTDLS    Dimensionless equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c         split the system in scalar equations by applying
c         Gaus-Seidel sweep
c
c         for all non virtual points do
c
c         calculate the constants in the non linear scalar equation for
c         the pressure in every gridpoint:
c               n                  n     2                       *
c   (f)   rho(p   + dpiter )-rho(p  ) -dt  * DG dpiter  = -dt D m
c               j         j        j                   j
c                 n
c         cons3=p
c                 j
c
c         cons2=DG    ( diagonal element DG operator)
c                 (j,j)
c                           n     2
c         cons1=dt*Dm-rho(p  ))+dt  * DG dpiter    - cons2*dpiter
c                           j                   j                 j
c         solve f(dpiter(j))=0 in the following steps:
c
c         1) specify an initial domain [a,b] for the zero finding algorithm
c         2) increase b and decrease a until sign(f(a))*sign(f(b))==0
c         3) apply the algorithm of Brent* to solve the non-linear equation
c
c         end loop over all points
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nconct   = infmat(6)
      iprincav = iinput(35)
      iprincom = iinput(34)
      iprintim = iinput(29)
      ipinfmat = infmat(13)
      dt       = rinput(iprintim   + 4 - 1)
      mscal    = iinput(iinput(9)  + 5 - 1)
      mcomupw  = iinput(iinput(9)  + 2 - 1)
      ni       = iinput(51)
      nj       = iinput(52)
      istartup = iinput(iinput(11) + 2 - 1)
      pres1tr  = rinput(iprincav + 3 -1)
      pres2tr  = rinput(iprincav + 4 -1)
      periodic = iinput(56)

      do k=1, mitergs

        if(mcomupw.eq.0) then

            if( mscal.eq.0) then

               do j = 1, nusol

                   matdp= 0d0
                   do iconct = 2, nconct

                      totshift(iconct)=infmat(ipinfmat-1+iconct)

                   end do

                   boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                         (mod(j,ni+1).eq. 1) .or.
     +                         (         j .le.ni) .or.
     +                         (         j .gt.((ni+1)*(nj-1))))
                   if( periodic.eq.1 .and. boundaryp ) then

                      call isgsper(iinmtvc(1), iinmtvc(4), iinmtvc(5),
     +                    iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +                    nsubfaces,
     +                    subfaces, totshift, j)
                   end if

                   do 220 iconct = 2, nconct

                      ishift = infmat(ipinfmat-1+iconct)

                      if(j+ishift.ge.1.and.j+ishift.le.nusol) then

                         matdp = matdp + matrix(nusol*(iconct-1)+j) *
     +                          dpiter(j+totshift(iconct))
                      end if

220                continue

                   checklin = -dt*(-rhs(j)+matdp)/
     +                    (dt*matrix(j)+jac(j)*
     +                       isdstlss(solut(j),rinput(iprincom),
     +                                 istartup, rinput(iprincav)))
                   if(.not.((solut(j).lt.pres1tr.and.
     +                        solut(j)+checklin.lt.pres1tr).or.
     +             (solut(j).gt.pres2tr.and.
     +                        solut(j)+checklin.gt.pres2tr))) then

                      cons3nl=  solut(j)
                      cons2nl=  dt*matrix(j)/jac(j)

                      matdp=matdp-matrix(j)*dpiter(j)
                          cons1nl= (dt*(-rhs(j)+matdp) -
     +                               jac(j)*isstdls(cons3nl,
     +                  rinput(iprincom), istartup,rinput(iprincav)))
     +                                                      /jac(j)

                      if(abs(cons3nl).gt.0) then
                        ddp=max(0.001d0,0.001d0*abs(cons3nl))
                      else
                        ddp=0.001d0
                      end if

                      dprside=dpiter(j)-0.1*ddp
                      dplside=dpiter(j)-0.1*ddp

                      l=1

  5                   if ((isstdls(cons3nl+dprside,rinput(iprincom),
     +                                   istartup, rinput(iprincav))+
     +                                   cons2nl*dprside + cons1nl)*
     +                     (isstdls(cons3nl+dplside,rinput(iprincom),
     +                                   istartup, rinput(iprincav))+
     +                                   cons2nl*dplside + cons1nl)
     +                                        .gt.0d0 )  then

                        l=l+1
                        dprside=dpiter(j)-0.1d0*dble(l)**6d0*ddp
                        dplside=dpiter(j)+0.1d0*dble(l)**6d0*ddp

                      goto 5

                      end if

                      dpiter(j)=isbrzo(dprside, dplside, 1d-23,
     +                tolbsec, cons1nl, cons2nl, cons3nl,
     +                        rinput(iprincom), mscal,
     +                                 istartup, rinput(iprincav) )
                   else

                      dpiter(j)=checklin

                   end if

               end do

            else if( mscal.eq.1) then

               do j = 1, nusol
               if((mod(j,ni+1).ne.0).and.
     +           (j.lt.nj*(ni+1))) then

               matdp= 0d0
               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do
               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp ) then

                  call isgsper(iinmtvc(1), iinmtvc(4), iinmtvc(5),
     +                iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +                nsubfaces,
     +                subfaces, totshift, j)
               end if

               do 210 iconct = 2, nconct

                   if(j+totshift(iconct).ge.1.and.
     +                         j+totshift(iconct).lt.nj*(ni+1)) then

                       matdp = matdp + matrix(nusol*(iconct-1)+j) *
     +                          dpiter(j+totshift(iconct))

                   end if

 210            continue

                   checklin = -dt*(-rhs(j)+matdp)/
     +                    (dt*matrix(j)+jac(j)*
     +              isdstfll(solut(j),istartup, rinput(iprincav)))
                   if(.not.((solut(j).lt.pres1tr.and.
     +                        solut(j)+checklin.lt.pres1tr).or.
     +             (solut(j).gt.pres2tr.and.
     +                        solut(j)+checklin.gt.pres2tr))) then
                      cons3nl=  solut(j)
                      cons2nl=  dt*matrix(j)/jac(j)

                      cons1nl= (dt*(-rhs(j)+matdp) -
     +                               jac(j)*isstdfl(cons3nl,
     +                         istartup, rinput(iprincav)))/jac(j)

                      if(abs(cons3nl).gt.0) then
                         ddp=max(0.0001d0,0.0001d0*abs(cons3nl))
                      else
                         ddp=0.0001d0
                      end if

                      dprside=dpiter(j)-0.1*ddp
                      dplside=dpiter(j)-0.1*ddp

                      l=1

  6                   if ((isstdfl(cons3nl+dprside, istartup,
     +                                         rinput(iprincav))+
     +                                   cons2nl*dprside + cons1nl)*
     +                    (isstdfl(cons3nl+dplside, istartup,
     +                                         rinput(iprincav))+
     +                                   cons2nl*dplside + cons1nl)
     +                                        .gt.0d0 )  then

                        l=l+1

                        dprside=dpiter(j)-0.1d0*l**5d0*ddp
                        dplside=dpiter(j)+0.1d0*l**5d0*ddp

                      goto 6

                      end if

                      dpiter(j)=isbrzo(dprside, dplside, 1d-23, tolbsec,
     +                   cons1nl, cons2nl, cons3nl,
     +                       rinput(iprincom), mscal, istartup,
     +                                             rinput(iprincav) )
                   else

                      dpiter(j)=checklin

                   end if

               end if
               end do

            end if

        else if (mcomupw.eq.1) then

            if( mscal.eq.0) then

               do j = 1, nusol

                 if(mod(j,ni+1).ne.0.or.
     +           ((j-mod(j,ni+1))/(ni+1).eq.nj+1)) then

                    matdp= 0d0
                    do iconct = 2, nconct

                       totshift(iconct)=infmat(ipinfmat-1+iconct)

                    end do

                    boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                          (mod(j,ni+1).eq. 1) .or.
     +                          (         j .le.ni) .or.
     +                          (         j .gt.((ni+1)*(nj-1))))
                    if( periodic.eq.1 .and. boundaryp) then

                       call isgsper(iinmtvc(1), iinmtvc(4),
     +                     iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                     iinmtvc(12), nsubfaces, subfaces,
     +                     totshift, j)
                    end if

                    do iconct = 2, nconct

                        if(j+totshift(iconct).ge.1.and.
     +                              j+totshift(iconct).le.nusol) then

                            matdp = matdp + matrix(nusol*
     +                                  (iconct-1)+j)*
     +                                     dpiter(j+totshift(iconct))
                        end if

                    end do

                    cons3nl=  solut(j)
                    cons2nl=  dt*matrix(j)/(jac(j)+dt*
     +                                (densup(j,2,1)+densup(j,2,2)))

                    drhol=0d0
                    drhor=0d0
                    drhob=0d0
                    drhot=0d0

c                   --- make sure that the shift will not take the index out of
c                       --- the range 1-nusol
c                       by setting shift+j to 1 if shift+j lower than zero and
c                       equal
c                       tp nusol if shift+ j exceeds nusol

                    do iconct = 2, nconct

                     totshift(iconct)= max( 1,   totshift(iconct)+j)-j
                     totshift(iconct)= min(nusol,totshift(iconct)+j)-j

                    end do

                    drhol=isstdls(solut(j+totshift(5))+
     +                               dpiter(j+totshift(5)),
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))-
     +                    isstdls(solut(j+totshift(5))            ,
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))
                    drhor=isstdls(solut(j+totshift(6))+
     +                               dpiter(j+totshift(6)),
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))-
     +                    isstdls(solut(j+totshift(6))            ,
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))
                    drhob=isstdls(solut(j+totshift(3))+
     +                               dpiter(j+totshift(3)),
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))-
     +                    isstdls(solut(j+totshift(3))          ,
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))
                    drhot=isstdls(solut(j+totshift(8))+
     +                               dpiter(j+totshift(8)),
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))-
     +                    isstdls(solut(j+totshift(8))          ,
     +                                        rinput(iprincom),
     +                                istartup, rinput(iprincav))

                    checklin= -(dt*(-rhs(j)+matdp) +dt*
     +                       (drhol*densup(j,1,1)+
     +                                         drhor*densup(j,3,1)+
     +                        drhob*densup(j,1,2)+
     +                                         drhot*densup(j,3,2)))/
     +                                ((jac(j)+
     +                         dt*(densup(j,2,1)+densup(j,2,2)))*
     +                              ( isdstlss(solut(j),
     +                                rinput(iprincom),istartup,
     +                                             rinput(iprincav))
     +                                                    + cons2nl))

                    if(.not.((solut(j).lt.pres1tr.and.
     +                            solut(j)+checklin.lt.pres1tr).or.
     +                 (solut(j).gt.pres2tr.and.
     +                           solut(j)+checklin.gt.pres2tr))) then

                      cons1nl= (dt*(-rhs(j)+matdp) +dt*
     +                         (drhol*densup(j,1,1)+
     +                                          drhor*densup(j,3,1) +
     +                          drhob*densup(j,1,2)+
     +                                          drhot*densup(j,3,2))-
     +                        (jac(j)+dt*(densup(j,2,1)+
     +                                           densup(j,2,2)))*
     +                                             isstdls(cons3nl,
     +                                  rinput(iprincom),
     +                                  istartup, rinput(iprincav)))/
     +                                  (jac(j)+
     +                            dt*(densup(j,2,1)+densup(j,2,2)))

                      if (abs(cons3nl).gt.0) then

                        ddp=max(0.0001d0,0.0001d0*abs(cons3nl))

                      else

                        ddp=0.0001d0

                      end if

                      dprside=dpiter(j)-0.1*ddp
                      dplside=dpiter(j)-0.1*ddp

                      l=1

  15                   if ((isstdls(cons3nl+dprside,rinput(iprincom),
     +                                   istartup, rinput(iprincav))+
     +                                   cons2nl*dprside + cons1nl)*
     +                     (isstdls(cons3nl+dplside,rinput(iprincom),
     +                                   istartup, rinput(iprincav))+
     +                                   cons2nl*dplside + cons1nl)
     +                                        .gt.0d0 )  then

                        l=l+1
                        dprside=dpiter(j)-0.1d0*dble(l)**6d0*ddp
                        dplside=dpiter(j)+0.1d0*dble(l)**6d0*ddp

                      goto 15

                      end if

                      dpiter(j)=isbrzo(dprside, dplside, 1d-23, tolbsec,
     +                                 cons1nl,     cons2nl,    cons3nl,
     +                                 rinput(iprincom), mscal,
     +                                      istartup , rinput(iprincav))
                    else

                      dpiter(j)=checklin

                    end if
                 end if
               end do

            else if( mscal.eq.1) then

               do j = 1, nusol

                 if( (mod(j,ni+1).ne.0) .and.
     +                (j.lt.nj*(ni+1)) ) then

                    matdp= 0d0
                    do iconct = 2, nconct

                       totshift(iconct)=infmat(ipinfmat-1+iconct)

                    end do

                    boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                          (mod(j,ni+1).eq. 1) .or.
     +                          (         j .le.ni) .or.
     +                          (         j .gt.((ni+1)*(nj-1))))
                    if( periodic.eq.1 .and. boundaryp) then

                       call isgsper(iinmtvc(1), iinmtvc(4),
     +                     iinmtvc(5), iinmtvc(10), iinmtvc(11),
     +                     iinmtvc(12), nsubfaces, subfaces,
     +                     totshift, j)
                    end if

                    do iconct = 2, nconct

                        if(j+totshift(iconct).ge.1.and.
     +                              j+totshift(iconct).le.nusol) then

                            matdp = matdp + matrix(nusol*
     +                                  (iconct-1)+j)*
     +                                     dpiter(j+totshift(iconct))
                        end if

                    end do

                    cons3nl=  solut(j)
                    cons2nl=  dt*matrix(j)/(jac(j)+dt*
     +                                (densup(j,2,1)+densup(j,2,2)))
                    drhol=0d0
                    drhor=0d0
                    drhob=0d0
                    drhot=0d0

c                   --- make sure that the shift will not take the index out of
c                       --- the range 1-nusol
c                       by setting shift+j to 1 if shift+j lower than zero and
c                       equal
c                       tp nusol if shift+ j exceeds nusol

                    do iconct = 2, nconct

                     totshift(iconct)= max( 1,   totshift(iconct)+j)-j
                     totshift(iconct)= min(nusol,totshift(iconct)+j)-j

                    end do

c                   --- left point

                    ppold=solut(j+totshift(5))
                    dpold=dpiter(j+totshift(5))

                    if( ppold.gt.pres2tr .and.
     +                        ppold + dpold.gt.pres2tr) then
                       drhol=isdstfll(ppold, istartup,
     +                              rinput(iprincav))*dpold
                    else
                       drhol=isstdfl(ppold+dpold,
     +                                   istartup, rinput(iprincav))-
     +                    isstdfl(ppold,istartup, rinput(iprincav))
                    end if

c                   --- right point

                    ppold=solut(j+totshift(6))
                    dpold=dpiter(j+totshift(6))
                    if( ppold.gt.pres2tr .and.
     +                        ppold + dpold.gt.pres2tr) then
                       drhor=isdstfll(ppold, istartup,
     +                              rinput(iprincav))*dpold
                    else
                       drhor=isstdfl(ppold+dpold,
     +                                   istartup, rinput(iprincav))-
     +                    isstdfl(ppold,istartup, rinput(iprincav))
                    end if

c                   --- bottom point

                    ppold=solut(j+totshift(3))
                    dpold=dpiter(j+totshift(3))

                    if( ppold.gt.pres2tr .and.
     +                        ppold + dpold.gt.pres2tr) then
                       drhob=isdstfll(ppold, istartup,
     +                              rinput(iprincav))*dpold
                    else
                       drhob=isstdfl(ppold+dpold,
     +                                   istartup, rinput(iprincav))-
     +                    isstdfl(ppold,istartup, rinput(iprincav))
                    end if

c                   --- top point

                    ppold=solut(j+totshift(8))
                    dpold=dpiter(j+totshift(8))

                    if( ppold.gt.pres2tr .and.
     +                        ppold + dpold.gt.pres2tr) then
                       drhot=isdstfll(ppold, istartup,
     +                              rinput(iprincav))*dpold
                    else
                       drhot=isstdfl(ppold+dpold,
     +                                   istartup, rinput(iprincav))-
     +                    isstdfl(ppold,istartup, rinput(iprincav))
                    end if

                    checklin= -(dt*(-rhs(j)+matdp) +dt*
     +                       (drhol*densup(j,1,1)+
     +                                         drhor*densup(j,3,1)+
     +                        drhob*densup(j,1,2)+
     +                                         drhot*densup(j,3,2)))/
     +                                ((jac(j)+
     +                         dt*(densup(j,2,1)+densup(j,2,2)))*
     +                              ( isdstfll(solut(j),
     +                                    istartup, rinput(iprincav))
     +                                                    + cons2nl))

                    if(.not.((solut(j).lt.pres1tr.and.
     +                           solut(j)+checklin.lt.pres1tr).or.
     +                 (solut(j).gt.pres2tr.and.
     +                           solut(j)+checklin.gt.pres2tr))) then
                        cons1nl= (dt*(-rhs(j)+matdp) +dt*
     +                           (drhol*densup(j,1,1)+
     +                                           drhor*densup(j,3,1)+
     +                            drhob*densup(j,1,2)+
     +                                           drhot*densup(j,3,2))-
     +                          (jac(j)+dt*(densup(j,2,1)+
     +                                              densup(j,2,2)))*
     +                                               isstdfl(cons3nl,
     +                                  istartup, rinput(iprincav)))/
     +                                  (jac(j)+
     +                             dt*(densup(j,2,1)+densup(j,2,2)))

                       if (abs(cons3nl).gt.0) then

                         ddp=max(0.0001d0,0.0001d0*abs(cons3nl))

                       else

                         ddp=0.0001d0

                       end if

                       dprside=dpiter(j)-0.1*ddp
                       dplside=dpiter(j)-0.1*ddp

                       l=1

  16                   if ((isstdfl(cons3nl+dprside, istartup,
     +                                              rinput(iprincav))+
     +                                     cons2nl*dprside + cons1nl)*
     +                        (isstdfl(cons3nl+dplside, istartup,
     +                                              rinput(iprincav))+
     +                                     cons2nl*dplside + cons1nl)
     +                                            .gt.0d0 )  then

                            l=l+1

                            dprside=dpiter(j)-0.1d0*l**5d0*ddp
                            dplside=dpiter(j)+0.1d0*l**5d0*ddp

                          goto 16

                       end if

                       dpiter(j)=isbrzo(dprside, dplside, 1d-23,
     +                               tolbsec, cons1nl, cons2nl,
     +                               cons3nl, rinput(iprincom),
     +                               mscal, istartup,
     +                                        rinput(iprincav) )

                    else

                       dpiter(j)=checklin

                    end if

                 end if

              end do

            end if

        end if

      end do

      end
