      subroutine isgsper(iblock, ni, nj, nblocks, nfaces, maxnsubf,
     +                    nsubfaces, subfaces, totshift, j)
c ======================================================================
c
c     programmer   Duncan van der Heul
c
c     version      1.0  7-5-1999
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    regulate the shift of the unknown for the nonlinear Gauss-Seidel
c    algorithm in case of periodical boundary conditions. Only
c    implementation for the case when the connection is made across
c    identical running indices, an j= constant face connected to an
c    j = constant face , or an i =constant face connected to an i is
c    constant face.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks,  nfaces, ni, nj,
     +        nsubfaces(1:nblocks, 1:nfaces),
     +        subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *),
     +        totshift(*), j

c     iblock    i    Block sequence number
c     j         i    General loop variable
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     subfaces  i    reserved
c     totshift  i    total shift of diagonals
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto(4), iface, isubface, bpi, ishift(9), jshift(9),
     +        idir, ihighneig, ilowneig, jhigh, jhighdef,
     +        jhighneig, jlow, jlowdef, jlowneig, mapdir, neigface,
     +        neigsface, ilowdef, ihighdef, ihigh, ilow, iorient
      logical ininterv

c     bpi       boundary index
c     ibto      Target block
c     idir      General loop variable for 1...ndim loop
c     iface     Face number
c     ihigh     Upper index of subface
c     ihighdef  Definite upper index of subface i direction
c     ihighneig Upper index of neighbouring subface i direction
c     ilow      Lower index of subface
c     ilowdef   Definite lower index of subface i direction
c     ilowneig  Lower index of neighbouring subface i direction
c     ininterv  Logical, .true. of boundary point belongs to subface
c               under consideration
c     iorient   Relative orientation of subfaces, see itopol
c     ishift    Shift in i-direction
c     isubface  Subface number
c     jhigh     Upper j-index for a subface
c     jhighdef  Definite upper index of subface j direction
c     jhighneig Upper index of neighbouring subface j direction
c     jlow      Lower j-index for a subface
c     jlowdef   Definite lower index of subface j direction
c     jlowneig  Lower index of neighbouring subface j direction
c     jshift    Shift in j-direction
c     mapdir    Direction of shift mapping
c     neigface  Index of neighbouring face
c     neigsface Index of neighbouring subface
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- determine which faces are internal to the domain

      do iface=1, nfaces
        ibto(iface)= 0
        do isubface = 1, nsubfaces(iblock, iface)
          ibto(iface) = max(ibto(iface),
     +             subfaces(iblock, iface, isubface, 5))
        end do

      end do

c     --- determine if a point is a boundary point, bpi indicates
c         what the position of the face is

      bpi = 0

      if( mod(j, ni+1).eq.1)  bpi = 1
      if( mod(j, ni+1).eq.ni) bpi = 2
      if( j.gt.(nj-1)*(ni+1) .and. j.lt. ((ni+1)*nj) )   bpi = 4
      if( j.le.ni)               bpi = 3

c     --- take care of the corner points, they have to be put in the
c          faces with the periodical boundary condition

c         first corner point j=1, normally in face 1, but if face 3 is
c         periodical
c         in face 3

       if(j.eq.1) bpi = 1
       if(j.eq.1 .and. ibto(3).ne.0) bpi = 3

c      --- second corner point j=ni, normally in face 2, but if face 3 is
c          --- periodical
c          in face 3

       if(j.eq.ni) bpi = 2
       if(j.eq.ni .and. ibto(3).ne.0) bpi = 3

c      --- third corner point j=(nj-1)*(ni+1)+1, normally in face 1,
c          but if face 4 is periodical
c          in face 4

       if(j.eq.((nj-1)*(ni+1)+1)) bpi = 1
       if(j.eq.((nj-1)*(ni+1)+1) .and. ibto(4).ne.0) bpi = 4

c      --- fourth corner point j=(nj-1)*(ni+1)+ni, normally in face 2,
c          but if face 3 is periodical
c          in face 3

       if(j .eq.((nj-1)*(ni+1)+ni)) bpi = 2
       if(j .eq.((nj-1)*(ni+1)+ni) .and. ibto(4).ne.0) bpi = 4

c      --- if the point under consideration is a boundary point and the face
c          contains a periodical boundary condition , special action has to be
c          taken, otherwise standard matrix vector multiplication

      if(ibto(bpi).ne.0 .and. bpi.ne.0) then

c     --- the point lies on a face with a periodical part. Determine the
c         interval boundaries of the defining subfaces of the face and
c         check if this point lies on a periodical subface

          if( bpi.eq.1) then

c         --- left face of the block, special treatment for connections
c              2, 5 and 7

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                jlow  = subfaces(iblock, bpi, isubface, 1)
                jhigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( (j/(ni+1)+1).ge.(jlow-1) .and.
     +                          (j/(ni+1)+1).le.(jhigh+1) )

                if( ininterv ) then

c               --- if in interval then save the end points of the interval

                   ilowdef  = 1
                   ihighdef = ilowdef
                   jlowdef  = jlow
                   jhighdef = jhigh

c                  --- if in interval then obtain the neigbouring face and
c                      --- subface

                   neigface=
     +                     subfaces(iblock, bpi, isubface, 6)
                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

c                  --- obtain the starting and end points of the neigbouring
c                      subface

                   if(neigface.ne.0)  then
                   if(neigface.eq.1)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       ilowneig = 1
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.2)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       ilowneig = ni
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.3)  then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=1
                       jhighneig=jlowneig
                   end if
                   if(neigface.eq.4) then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=nj
                       jhighneig=jlowneig
                   end if

c                  --- the interface is coupled over the largest interval of
c                       ihighneig-ilowneig and jhighneig-jlowneig
c                        assume it is the same as target otherwise change

                   idir=1
                   if( abs( ihighneig-ilowneig).gt.
     +                           abs( jhighneig-jlowneig) ) idir = 2

c                  --- determine the jshift the ishift and both directions

                   if( idir.eq.1) then

c                  --- blocks are connected over same running index

                      iorient=subfaces(iblock, bpi, isubface, 8)
                      mapdir = 1
                      if( iorient.eq.1 .or. iorient.eq.3 .or.
     +                       iorient.eq.5 .or. iorient.eq.7)
     +                          mapdir= -1
                      if( mapdir.gt.0) then
                        jshift(2)= jlowneig-jlowdef-1
                        jshift(5)= jlowneig-jlowdef
                        jshift(7)= jlowneig-jlowdef+1
                      else if ( mapdir.lt.0) then
                        jshift(2)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)+1
                        jshift(5)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)
                        jshift(7)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)-1
                      end if
                      ishift(2) =ilowneig-ilowdef
                      ishift(5) =ilowneig-ilowdef
                      ishift(7) =ilowneig-ilowdef
                      totshift(2)=jshift(2)*(ni+1)+ishift(2)
                      totshift(5)=jshift(5)*(ni+1)+ishift(5)
                      totshift(7)=jshift(7)*(ni+1)+ishift(7)
                   else if( idir.eq.2) then

c                  --- blocks are connected over different running index
c                                      currently not implemented  !!!!

                   end if
                   end if

                end if

             end do

          end if

          if( bpi.eq.2) then

c         --- right face of the block, special treatment for connections
c              4, 6 and 9

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                jlow  = subfaces(iblock, bpi, isubface, 1)
                jhigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( (j/(ni+1)+1).ge.(jlow-1) .and.
     +                         (j/(ni+1)+1).le.(jhigh+1) )

                if( ininterv ) then

c               --- if in interval then save the end points of the interval

                   ilowdef  = ni
                   ihighdef = ilowdef
                   jlowdef  = jlow
                   jhighdef = jhigh

c                  --- if in interval then obtain the neigbouring face and
c                      --- subface

                   neigface=
     +                     subfaces(iblock, bpi, isubface, 6)
                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

c                  --- obtain the starting and end points of the neigbouring
c                      subface

                   if(neigface.ne.0)  then
                   if(neigface.eq.1)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       ilowneig = 1
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.2)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       ilowneig = ni
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.3)  then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=1
                       jhighneig=jlowneig
                   end if
                   if(neigface.eq.4) then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=nj
                       jhighneig=jlowneig
                   end if

c                  --- the interface is coupled over the largest interval of
c                       ihighneig-ilowneig and jhighneig-jlowneig
c                        assume it is the same as target otherwise change

                   idir=1
                   if( abs( ihighneig-ilowneig).gt.
     +                           abs( jhighneig-jlowneig) ) idir = 2

c                  --- determine the jshift the ishift and both directions

                   if( idir.eq.1) then

c                  --- blocks are connected over same running index

                      iorient=subfaces(iblock, bpi, isubface, 8)
                      mapdir = 1
                      if( iorient.eq.1 .or. iorient.eq.3 .or.
     +                       iorient.eq.5 .or. iorient.eq.7)
     +                          mapdir= -1
                      if( mapdir.gt.0) then
                        jshift(4)= jlowneig-jlowdef-1
                        jshift(6)= jlowneig-jlowdef
                        jshift(9)= jlowneig-jlowdef+1
                      else if ( mapdir.lt.0) then
                        jshift(4)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)+1
                        jshift(6)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)
                        jshift(9)= -2*(j/(ni+1)+1)+
     +                  (jhighdef*jhighneig-jlowdef*jlowneig)/
     +                           (jhighdef-jlowdef)-1
                      end if
                      ishift(4) =ilowneig-ilowdef
                      ishift(6) =ilowneig-ilowdef
                      ishift(9) =ilowneig-ilowdef
                      totshift(4)=jshift(4)*(ni+1)+ishift(4)
                      totshift(6)=jshift(6)*(ni+1)+ishift(6)
                      totshift(9)=jshift(9)*(ni+1)+ishift(9)
                   else if( idir.eq.2) then

c                  --- blocks are connected over different running index
c                                      currently not implemented  !!!!

                   end if
                   end if

                end if

             end do

          end if
          if( bpi.eq.3) then

c         --- lower face of the block, special treatment for connections
c              2, 3 and 4

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                ilow  = subfaces(iblock, bpi, isubface, 1)
                ihigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( j.ge.(ilow-1) .and.
     +                          j.le.(ihigh+1) )

                if( ininterv ) then

c               --- if in interval then save the end points of the interval

                   ilowdef  = ilow
                   ihighdef = ihigh
                   jlowdef  = 1
                   jhighdef = jlowdef

c                  --- if in interval then obtain the neigbouring face and
c                      --- subface

                   neigface=
     +                     subfaces(iblock, bpi, isubface, 6)
                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

c                  --- obtain the starting and end points of the neigbouring
c                      subface

                   if(neigface.ne.0) then
                   if(neigface.eq.1)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 3)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 4)
                       ilowneig = 1
                       ihighneig =ilowneig
                   else if(neigface.eq.2)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 3)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 4)
                       ilowneig = ni
                       ihighneig =ilowneig
                   else if(neigface.eq.3)  then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=1
                       jhighneig=jlowneig
                   else if(neigface.eq.4) then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=nj
                       jhighneig=jlowneig
                   end if

c                  --- the interface is coupled over the largest interval of
c                       ihighneig-ilowneig and jhighneig-jlowneig
c                        assume it is the same as target otherwise change

                   idir=1
                   if( abs( jhighneig-jlowneig).gt.
     +                           abs( ihighneig-ilowneig) ) idir = 2

c                  --- determine the jshift the ishift and both directions

                   if( idir.eq.1) then

c                  --- blocks are connected over same running index

                      iorient=subfaces(iblock, bpi, isubface, 8)
                      mapdir = 1
                      if( iorient.eq.1 .or. iorient.eq.3 .or.
     +                       iorient.eq.5 .or. iorient.eq.7)
     +                          mapdir= -1
                      if( mapdir.gt.0) then
                        ishift(2)= ilowneig-ilowdef-1
                        ishift(3)= ilowneig-ilowdef
                        ishift(4)= ilowneig-ilowdef+1
                      else if ( mapdir.lt.0) then
                        ishift(2)= -2*j+1+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)
                        ishift(3)= -2*j+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)
                        ishift(4)= -2*j-1+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)
                      end if
                      jshift(2) =jlowneig-jlowdef
                      jshift(3) =jlowneig-jlowdef
                      jshift(4) =jlowneig-jlowdef
                      totshift(2)=jshift(2)*(ni+1)+ishift(2)
                      totshift(3)=jshift(3)*(ni+1)+ishift(3)
                      totshift(4)=jshift(4)*(ni+1)+ishift(4)
                   else if( idir.eq.2) then

c                  --- blocks are connected over different running index
c                                      currently not implemented  !!!!

                   end if
                   end if

                end if

             end do

          end if
          if( bpi.eq.4) then

c         --- upper face of the block, special treatment for connections
c              7, 8 and 9

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                ilow  = subfaces(iblock, bpi, isubface, 1)
                ihigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( mod(j,ni+1).ge.(ilow-1) .and.
     +                           mod(j,ni+1).le.(ihigh+1) )

                if( ininterv ) then

c               --- if in interval then save the end points of the interval

                   ilowdef  = ilow
                   ihighdef = ihigh
                   jlowdef  = nj
                   jhighdef = jlowdef

c                  --- if in interval then obtain the neigbouring face and
c                      --- subface

                   neigface=
     +                     subfaces(iblock, bpi, isubface, 6)
                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

c                  --- obtain the starting and end points of the neigbouring
c                      subface

                   if(neigface.ne.0) then
                   if(neigface.eq.1)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 3)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 4)
                       ilowneig = 1
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.2)  then
                       jlowneig =
     +                      subfaces(iblock, neigface, neigsface, 3)
                       jhighneig =
     +                      subfaces(iblock, neigface, neigsface, 4)
                       ilowneig = ni
                       ihighneig =ilowneig
                   end if
                   if(neigface.eq.3)  then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=1
                       jhighneig=jlowneig
                   end if
                   if(neigface.eq.4) then
                       ilowneig  =
     +                      subfaces(iblock, neigface, neigsface, 1)
                       ihighneig =
     +                      subfaces(iblock, neigface, neigsface, 2)
                       jlowneig=nj
                       jhighneig=jlowneig
                   end if

c                  --- the interface is coupled over the largest interval of
c                       ihighneig-ilowneig and jhighneig-jlowneig
c                        assume it is the same as target otherwise change

                   idir=1
                   if( abs( jhighneig-jlowneig).gt.
     +                           abs( ihighneig-ilowneig) ) idir = 2

c                  --- determine the jshift the ishift and both directions

                   if( idir.eq.1) then

c                  --- blocks are connected over same running index

                      iorient=subfaces(iblock, bpi, isubface, 8)
                      mapdir = 1
                      if( iorient.eq.1 .or. iorient.eq.3 .or.
     +                       iorient.eq.5 .or. iorient.eq.7)
     +                          mapdir= -1
                      if( mapdir.gt.0) then
                        ishift(7)= ilowneig-ilowdef-1
                        ishift(8)= ilowneig-ilowdef
                        ishift(9)= ilowneig-ilowdef+1
                      else if ( mapdir.lt.0) then
                        ishift(7)= -2*mod(j,ni+1)+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)+1
                        ishift(8)= -2*mod(j,ni+1)+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)
                        ishift(9)= -2*mod(j,ni+1)+
     +                  (ihighdef*ihighneig-ilowdef*ilowneig)/
     +                           (ihighdef-ilowdef)-1
                      end if
                      jshift(7) =jlowneig-jlowdef
                      jshift(8) =jlowneig-jlowdef
                      jshift(9) =jlowneig-jlowdef
                      totshift(7)=jshift(7)*(ni+1)+ishift(7)
                      totshift(8)=jshift(8)*(ni+1)+ishift(8)
                      totshift(9)=jshift(9)*(ni+1)+ishift(9)
                   else if( idir.eq.2) then

c                  --- blocks are connected over different running index
c                                      currently not implemented  !!!!

                   end if
                   end if

                end if

             end do

          end if

      end if

      end
